/**
 * SwarmScreen Project
 *
 * File:         Main.java
 * RCS:          $Id: Main.java,v 1.10 2009/04/16 17:59:17 drc915 Exp $
 * Description:  Main class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 30, 2006 at 7:31:14 AM
 * Language:     Java
 * Package:      edu.northwestern.swarmscreen
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2008, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.swarmscreen;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;

import org.gudy.azureus2.core3.config.COConfigurationManager;
import org.gudy.azureus2.core3.logging.LogAlert;
import org.gudy.azureus2.core3.logging.Logger;
import org.gudy.azureus2.core3.util.Debug;
import org.gudy.azureus2.plugins.PluginException;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.PluginListener;
import org.gudy.azureus2.plugins.UnloadablePlugin;
import org.gudy.azureus2.plugins.clientid.ClientIDManager;
import org.gudy.azureus2.plugins.config.ConfigParameter;
import org.gudy.azureus2.plugins.config.ConfigParameterListener;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadListener;
import org.gudy.azureus2.plugins.download.DownloadManager;
import org.gudy.azureus2.plugins.logging.LoggerChannel;
import org.gudy.azureus2.plugins.logging.LoggerChannelListener;
import org.gudy.azureus2.plugins.network.ConnectionManager;
import org.gudy.azureus2.plugins.peers.Peer;
import org.gudy.azureus2.plugins.ui.UIInstance;
import org.gudy.azureus2.plugins.ui.UIManager;
import org.gudy.azureus2.plugins.ui.UIManagerListener;
import org.gudy.azureus2.plugins.ui.config.BooleanParameter;
import org.gudy.azureus2.plugins.ui.config.IntParameter;
import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
import org.gudy.azureus2.plugins.ui.model.BasicPluginViewModel;
import org.gudy.azureus2.plugins.update.UpdateCheckInstance;
import org.gudy.azureus2.plugins.utils.LocaleUtilities;
import org.gudy.azureus2.plugins.utils.UTTimer;
import org.gudy.azureus2.plugins.utils.UTTimerEvent;
import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;
import org.gudy.azureus2.plugins.utils.Utilities;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderException;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderFactory;
import org.gudy.azureus2.pluginsimpl.local.utils.resourcedownloader.ResourceDownloaderFactoryImpl;
import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;

import edu.northwestern.swarmscreen.analysis.Driver;
import edu.northwestern.swarmscreen.connection.ConnectionProfiler;
import edu.northwestern.swarmscreen.torrents.TorrentFetcher;
import edu.northwestern.swarmscreen.torrents.TorrentManager;
import edu.northwestern.swarmscreen.ui.SSView;

//import com.azureus.plugins.aznetmon.main.RSTPacketCountPerformer;
//import com.azureus.plugins.aznetmon.main.RSTPacketStats;
/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The Main class launches this plugin.
 */
public class Main implements UnloadablePlugin, DownloadListener  {


	
	


	/** whether UI is created yet */
	boolean isCreated = false; 
	static UISWTInstance swtInstance = null;
	  UISWTViewEventListener myView = null;
	  
    static org.gudy.azureus2.plugins.PluginInterface pi_az = null;

    DownloadManager dm;
    ConnectionManager cm;
    ClientIDManager cidm;
    Utilities ut;
    BasicPluginViewModel model;
    UpdateCheckInstance uci;
    private static LoggerChannel log;
    private BooleanParameter enableStatsParam;
	private UTTimer torrentCheckTimer;
	private UIManagerListener uiListener;
	BasicPluginConfigModel config_model;
	private LoggerChannelListener logListener;
	private IntParameter reportingTimeoutParam;
	private IntParameter maxPingsParam;
	private Properties props;

    /** whether to use plugin-specific download-only torrent */
    public static final boolean USE_DO_TORRENT = false;
	protected static final boolean RUN_PERFORMACE_TESTS = false;

	HashMap<String, UTTimer> timers = new HashMap<String, UTTimer>();

    /** downloader factory */
    protected static ResourceDownloaderFactory rd_factory = ResourceDownloaderFactoryImpl.getSingleton();

    

    private UTTimer timer;
	protected boolean canCheckPlugins = false;
	private TorrentManager tm;
	private static boolean isShuttingDown = false;
	//private RSTPacketCountPerformer perf;
	
	private static ArrayList<String> propertiesLocations = new ArrayList<String>();
	private static Main self;
    
    public Main(){
    	super();
    }

	public void loadProperties() {
		props = new Properties();

    	//try {

    	// here's the name of the file

    	boolean found = false;
    	try {
    		for (String location : propertiesLocations){
    			File properties_file = new File(location);
    			// if properties file exists on its own then override any properties file
    			// potentially held within a jar
    			if (properties_file.exists()) {
    				FileInputStream fis = null;

    				try {
    					fis = new FileInputStream(properties_file);
    					props.load(fis);					
    					found = true;
    					break;
    				} finally {
    					if (fis != null) {
    						fis.close();
    					}
    				}
    			}
    		}
    		if (!found) {
    			// properties file wasn't there, so use the jar
    			ClassLoader classLoader = pi_az.getPluginClassLoader();

    			if (classLoader instanceof URLClassLoader) {
    				URLClassLoader current = (URLClassLoader) classLoader;

    				URL url = current.findResource("ss.properties");

    				if (url != null) {
    					props.load(url.openStream());
    				} else {
    					throw (new Exception(
    							"failed to load ss.properties from jars"));
    				}
    			}
    		}
    	} catch (Throwable e) {
    		Debug.printStackTrace(e);

    		String msg = "Can't read 'ss.properties' for plugin '" +
    		pi_az.getPluginName() + "': file may be missing";

    		Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_ERROR,
    				msg));

    		System.out.println(msg);

    		try {
    			throw (new PluginException(msg, e));
    		} catch (PluginException e1) {
    			// TODO Auto-generated catch block
    			e1.printStackTrace();
    		}
    	}


	}

	/* (non-Javadoc)
     * @see org.gudy.azureus2.plugins.Plugin#initialize(org.gudy.azureus2.plugins.PluginInterface)
     */
    public void initialize(org.gudy.azureus2.plugins.PluginInterface arg0) throws PluginException {
    	self = this;
    	
    	pi_az = arg0;
    	
        // this loads in the internat stuff
        pi_az.getUtilities().getLocaleUtilities()
          .integrateLocalisedMessageBundle("edu.northwestern.swarmscreen.internat.Messages");
    	
        
		 log = pi_az.getLogger().getChannel("SwarmScreenLog"); 

		 arg0.addListener(new PluginListener(){

//			@Override
			public void closedownComplete() {
				// TODO Auto-generated method stub
				
			}

//			@Override
			public void closedownInitiated() {
				try {
					unload();
					TorrentManager.getInstance().deleteTransientTorrents();					
				} catch (PluginException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}

//			@Override
			public void initializationComplete() {
				canCheckPlugins  = true;

				if (RUN_PERFORMACE_TESTS){
					Driver.getInstance();
				}
				
				log.log("Initializing SwarmScreen version " + pi_az.getPluginVersion());
				

		        // register listeners for downloads      
		        tm = new TorrentManager(pi_az);
		        pi_az.getDownloadManager().addListener(tm);
		        getTimer("TorrentManager").addPeriodicEvent(1000*SSConfiguration.getInstance().getPrivacyUpdateInterval(), 
		        		tm);
		        getTimer("TorrentFetcher").addEvent(1000*SSConfiguration.getInstance().getTorrentDownloadInterval(), 
		        		TorrentFetcher.getInstance());
		        getTimer("ConnectionProfiler").addPeriodicEvent(
		        		1000*SSConfiguration.getInstance().getStatsUpdateTime(), 
		        		ConnectionProfiler.getInstance());

		        getTimer("SwarmScreenRateAdjuster").addPeriodicEvent(1*1000, 
		        		new UTTimerEventPerformer(){
					public void perform(UTTimerEvent event) {
						TorrentManager.getInstance().fixTransferRates();
						
					}});
			}});
        
        propertiesLocations.add(pi_az.getPluginDirectoryName() +
        		java.io.File.separator + "ss.properties");
        if (System.getProperties().getProperty("user.home")!=null){
	        propertiesLocations.add(System.getProperties().getProperty("user.home") +
	        		java.io.File.separator + "ss.properties");
        }
        
        // initialize properties
        loadProperties(); 
        SSConfiguration.getInstance().loadProperties(props);
        
        timer = pi_az.getUtilities().createTimer("GenericTimer");
        
        

 
        UIManager ui_manager = pi_az.getUIManager();

        // TODO add GUI options

       
        
//        enableStatsParam.addListener(
//                new ParameterListener()
//                {
//                    public void
//                    parameterChanged(
//                        Parameter   param )
//                    {
//                    	
//                    }
//                });
            
        model = pi_az.getUIManager().createBasicPluginViewModel("SwarmScreenView");

        model.getActivity().setVisible(false);
        model.getProgress().setVisible(false);

        // print output to log window
        logListener = new LoggerChannelListener() {
            public void messageLogged(int type, String message) {
                model.getLogArea().appendText(message + "\n");
            }

            public void messageLogged(String str, Throwable error) {
                model.getLogArea().appendText(error.toString() + "\n");
            }
        };
        
        log.addListener(logListener);
        
        uiListener = new UIManagerListener() {
            public void UIAttached(UIInstance instance) {
                if (instance instanceof UISWTInstance) {
                  swtInstance = (UISWTInstance)instance;
                  myView = new SSView(pi_az);
                  swtInstance.addView(UISWTInstance.VIEW_MAIN, SSView.VIEWID, myView);
                  
                }
              }

              public void UIDetached(UIInstance instance) {
                if (instance instanceof UISWTInstance) {
                	swtInstance = null;
                  
                }
              }
            };
            
        pi_az.getUIManager().addUIListener(uiListener);

        try {
		       String dir = COConfigurationManager.getDirectoryParameter("General_sDefaultTorrent_Directory");
		       if (dir==null || dir.length()<=0){
		    	   LocaleUtilities loc = Main.getPluginInterface().getUtilities().getLocaleUtilities();
		    	   log.logAlert(LoggerChannel.LT_ERROR, 
		    			   loc.getLocalisedMessageText("ss.noDefaultDir"));
		       }
        } catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }


    // from here on down, these are download listener actions.
    // can be used for net positioning control by adding/removing "nearby" peers
    public void stateChanged(Download download, int old_state, int new_state) {
        
    }

    public void positionChanged(Download download, int oldPosition,
        int newPosition) {
    }

    /* (non-Javadoc)
     * @see java.lang.Object#finalize()
     */
    @Override
    protected void finalize() throws Throwable {
        unload();
    }
    
    public UTTimer getTimer(){
    	return timer;
    }

    /* (non-Javadoc)
     * @see org.gudy.azureus2.plugins.UnloadablePlugin#unload()
     */
    public void unload() throws PluginException {
    
    	
    	try {
    	isShuttingDown  = true;
    	if (swtInstance!=null) swtInstance.removeViews(UISWTInstance.VIEW_MAIN, SSView.VIEWID);    	
    	pi_az.getUIManager().removeUIListener(uiListener);
    	model.destroy();
//    	config_model.destroy();
    	log.removeListener(logListener);
    	timer.destroy();
    	for (UTTimer t : timers.values()) t.destroy();
    	SSConfiguration.getInstance().writeProperties(new Properties(), propertiesLocations);


  
        
        if (torrentCheckTimer!=null) torrentCheckTimer.destroy();

        
    	} catch (Exception e){
    		e.printStackTrace();
    	}
    	
    }    
    
    public static void saveProperties(){
    	if (propertiesLocations!=null && SSConfiguration.getInstance()!=null)
    		SSConfiguration.getInstance().writeProperties(new Properties(), propertiesLocations);

    }
    


	/**
     * Checks for torrents that this node should connect to for the purpose of
     * getting peers for experiments.
     */
    public static void checkForTorrentToDownload() {
    	
    }
    
    
    public static int getPeerPort(String dest){
        // find the other guy's port via the custom torrent
        for (Download d : pi_az.getDownloadManager().getDownloads()) {
                if (d.getPeerManager() != null) {
                    for (Peer p : d.getPeerManager().getPeers()) {
                        if (p.getIp().contains(dest)) {
                            return p.getTCPListenPort();                            
                        } // end if
                    } // end for peers
                }

                return -1;
        } // end for downloads
        return -1;
    }
    
    public static Properties downloadFromURL(String url, int timeout) {
        InputStream is = null;
        ResourceDownloader dl = null;
        
		Properties details = new Properties();
        try {
			dl = rd_factory.create(new URL(url));
		

			dl = rd_factory.getTimeoutDownloader(dl, timeout);

	        // download the experiment list 
	        is = dl.download();
	        details.load(is);
	        dl.cancel();
        } catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ResourceDownloaderException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
        return details;
    }

	public static boolean isRunning() {
		return pi_az!=null;
	}

	public static PluginInterface getPluginInterface() {
		return pi_az;
	}
	

	public static LoggerChannel getLogger(){
		if (log==null){
			
		}
		return log;
	}

	public static void showView() {
		if (swtInstance!=null) swtInstance.openView(UISWTInstance.VIEW_MAIN, SSView.VIEWID, null);

	}


	public static ArrayList<String> getPropertiesLocations() {
		return propertiesLocations;
	}

	public static Main getInstance() {
		
		return self;
	}

	public UTTimer getTimer(String name) {
		if (timers.get(name)==null){
			timers.put(name, pi_az.getUtilities().createTimer(name));
		}
		return timers.get(name);
	}

	public static boolean isShuttingDown() {
		return isShuttingDown;
	}

}
