/**
 * SwarmScreen Project
 *
 * File:         NewsConfiguration.java
 * RCS:          $Id: SSConfiguration.java,v 1.9 2009/04/16 17:59:17 drc915 Exp $
 * Description:  NewsConfiguration class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 26, 2007 at 9:02:05 AM
 * Language:     Java
 * Package:      edu.northwestern.swarmscreen
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.swarmscreen;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Properties;
import java.util.UUID;
import java.util.Map.Entry;

//import com.azureus.plugins.aznetmon.main.RSTPacketStats;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The NewsConfiguration class ...
 */
public class SSConfiguration {
	private static final boolean DEBUG = false;

	private static SSConfiguration self;
	
	boolean recordStats = true;
	boolean sendToDatabase = true;
	int peerUpdateIntervalSec = 60;
	int digFrequencySec = 60;
	int vivaldiFrequencySec = 60;
	boolean doTraceroute = true;
	double cosSimThreshold = 0.2;
	int ratioUpdateFrequencySec = 600;
	int dbUpdateFreqSec = 3600;
	int updateCheckFreqSec = 43200;
	boolean doRemoteDig = true;
	int maxPeers = 100;
	String uuid = null;

	private int dbReconnectInterval = 15*60; // 10 minutes to retry

	private int reportingTimeout = 60;

	private int maxPings = 10;

	private boolean streamDebug = false;

	private boolean runExperiments;

	private boolean visualize=false;
	
	/** if true, will increase dig rate incrementally */
	private boolean adaptiveDig = false;
	
	/** initial dig interval (s) */
	private long digStart = 30;
	
	/** longest period between successive digs */
	private long digMaxInterval = 30*60;
	
	/** increment for dig interval */
	private long digIncrement = 60;

	private int dropVivaldiPct = 0;
	
	/** the sample interval for news, in seconds */
	private int sampleInterval = 30;

	private boolean override;

	private int alarmThreshold = 5;

	private int anomCountThreshold=3;

	private boolean onoRunning;

	private int pctDiffTrigger = 10;
	
	private boolean suppressWarnings = false;
	
	/** if true, override the user's settings with those of the server */
	private boolean globalOverride = false;

	/** time (s) between checks for privacy issues */
	private int privacyUpdateInterval = 15;

	/** time (s) between fetching of torrents to download */
	private int torrentDownloadInterval = 60*60; // 1 hr for now

	private int privacyLevel = 50;

	private String[] torrentURLs = new String[]{};

	private long torrentRotateInterval = 2*60*1000L;

	private boolean enabled = false;

	private String torrentDataDir;

	/** time (s) between updates to connection stats */
	private int statsUpdateTime = 5;

	/** max file size for random torrents, in MB */
	private int maxFileSize = 2*1000;
	

	public int getDbUpdateFreqSec() {
		return dbUpdateFreqSec;
	}


	public void setDbUpdateFreqSec(int dbUpdateFreqSec) {
		this.dbUpdateFreqSec = dbUpdateFreqSec;
	}


	public boolean isDoRemoteDig() {
		return doRemoteDig;
	}


	public void setDoRemoteDig(boolean doRemoteDig) {
		this.doRemoteDig = doRemoteDig;
	}


	public int getMaxPeers() {
		return maxPeers;
	}


	public void setMaxPeers(int maxPeers) {
		this.maxPeers = maxPeers;
	}


	public int getUpdateCheckFreqSec() {
		return updateCheckFreqSec;
	}


	public void setUpdateCheckFreqSec(int updateCheckFreqSec) {
		this.updateCheckFreqSec = updateCheckFreqSec;
	}


	public synchronized static SSConfiguration getInstance(){
		if (self==null){
			self = new SSConfiguration();
		}
		return self;
	}


	public double getCosSimThreshold() {
		return cosSimThreshold;
	}


	public void setCosSimThreshold(double cosSimThreshold) {
		this.cosSimThreshold = cosSimThreshold;
	}


	public int getDigFrequencySec() {
		return adaptiveDig ? (int)digStart : digFrequencySec;
	}


	public void setDigFrequencySec(int digFrequencySec) {
		this.digFrequencySec = digFrequencySec;
	}


	public boolean isDoTraceroute() {
		return doTraceroute;
	}


	public void setDoTraceroute(boolean doTraceroute) {
		this.doTraceroute = doTraceroute;
	}


	public int getPeerUpdateIntervalSec() {
		return peerUpdateIntervalSec;
	}


	public void setPeerUpdateIntervalSec(int peerUpdateIntervalSec) {
		this.peerUpdateIntervalSec = peerUpdateIntervalSec;
	}


	public int getRatioUpdateFrequencySec() {
		return ratioUpdateFrequencySec;
	}


	public void setRatioUpdateFrequencySec(int ratioUpdateFrequencySec) {
		this.ratioUpdateFrequencySec = ratioUpdateFrequencySec;
	}


	public boolean isRecordStats() {
		return recordStats;
	}


	public void setRecordStats(boolean recordStats) {
		this.recordStats = recordStats;
	}


	public boolean isSendToDatabase() {
		return sendToDatabase;
	}


	public void setSendToDatabase(boolean sendToDatabase) {
		this.sendToDatabase = sendToDatabase;
	}


	public int getVivaldiFrequencySec() {
		return vivaldiFrequencySec;
	}


	public void setVivaldiFrequencySec(int vivaldiFrequencySec) {
		this.vivaldiFrequencySec = vivaldiFrequencySec;
	}


	public void writeProperties(Properties props, ArrayList<String> locations) {
//		props.put("ss.enableStats", recordStats ? "true" : "false");
//		props.put("ss.override", override ? "true" : "false");
//		props.put("ss.stats.dbUpdate", dbUpdateFreqSec+"" );
//		props.put("ss.update.updateCheck", updateCheckFreqSec+"");
//		props.put("ss.digger.sleepTime", digFrequencySec+"");
//		props.put("ss.digger.maxPeers", maxPeers+"");
//		props.put("ss.digger.digForOthers", doRemoteDig+"");
//		props.put("ss.cossim", cosSimThreshold+"");
//		props.put("ss.doTraceroute", doTraceroute ? "true" : "false");
//		props.put("ss.stats.peerUpdate", peerUpdateIntervalSec+"");
//		props.put("ss.vivaldiUpdate", vivaldiFrequencySec+"");
//		props.put("ss.position.ratioUpdate", ratioUpdateFrequencySec+"");
//		props.put("ss.reportingTimeout", reportingTimeout+"");
//		props.put("ss.maxPings", maxPings+"");
//		props.put("ss.uuid", uuid);
//		props.put("ss.streamDebug", streamDebug ? "true" : "false");
//		props.put("ss.runExperiments", runExperiments ? "true" : "false");
//		props.put("ss.visualize", visualize ? "true" : "false");
//		props.put("ss.digger.adaptive", adaptiveDig ? "true" : "false");
//		props.put("ss.digger.digStartInterval", digStart+"");
//		props.put("ss.digger.digMaxInterval", digMaxInterval+"");
//		props.put("ss.digger.digIncrement", digIncrement+"");
//		props.put("ss.sampleInterval", sampleInterval+"");
//		props.put("ss.alarmThresh", alarmThreshold+"");
//		props.put("ss.anomCountLocal", anomCountThreshold+"");
//		props.put("ss.pctDiffTrigger", pctDiffTrigger+"");
//		props.put("ss.suppressWarnings", suppressWarnings ? "true" : "false");
		props.put("ss.globalOverride", globalOverride ? "true" : "false");
		props.put("ss.privacyUpdateInterval", privacyUpdateInterval+"");
		props.put("ss.torrentDownloadInterval", torrentDownloadInterval+"");
		props.put("ss.privacyLevel", privacyLevel+"");
		props.put("ss.torrentRotateInterval", torrentRotateInterval+"");
		props.put("ss.enabled", enabled ? "true" : "false");
		props.put("ss.torrentDataDir", torrentDataDir+"");
		props.put("ss.maxFileSize", maxFileSize+"");
		int i = 1;
		for (String url : torrentURLs){
			props.put("ss.torrentURL."+i, url+"");
			i++;
		}
		for (String filename : locations){
			try {
				FileOutputStream fos = new FileOutputStream(filename);
				props.store(fos, "");
				break;
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}


	public void loadProperties(Properties props) {

		ArrayList<String> tempURLs = new ArrayList<String>();
		
		for (Entry<Object, Object> ent : props.entrySet()){
			try {
			String key = (String)ent.getKey();
			String value = (String)ent.getValue();
			if (key.equals("ss.enableStats")){
				recordStats= value.equals("true") ? true : false;
			} else if (key.equals("ss.stats.dbUpdate")){
				dbUpdateFreqSec = Integer.valueOf(value);
			} else if (key.equals("ss.update.updateCheck")){
				updateCheckFreqSec = Integer.valueOf(value);
			}else if (key.equals("ss.digger.sleepTime")){
				digFrequencySec= Integer.valueOf(value);
			}else if (key.equals("ss.digger.maxPeers")){
				maxPeers = Integer.valueOf(value);
			}else if (key.equals("ss.digger.digForOthers")){
				doRemoteDig= value.equals("true") ? true : false;
			}else if (key.equals("ss.cossim")){
				cosSimThreshold = Double.parseDouble(value);
			}else if (key.equals("ss.doTraceroute")){
				doTraceroute= value.equals("true") ? true : false;
			}else if (key.equals("ss.stats.peerUpdate")){
				peerUpdateIntervalSec = Integer.valueOf(value);
			}else if (key.equals("ss.vivaldiUpdate")){
				vivaldiFrequencySec = Integer.valueOf(value);
			}else if (key.equals("ss.position.ratioUpdate")){
				ratioUpdateFrequencySec = Integer.valueOf(value);
			}else if (key.equals("ss.reportingTimeout")){
				reportingTimeout = Integer.valueOf(value);
			} else if (key.equals("ss.maxPings")){
				maxPings = Integer.valueOf(value);
			} else if (key.equals("ss.uuid")){
				uuid = value;
			} else if (key.equals("ss.streamDebug")){
				streamDebug = value.equals("true") ? true : false;;
			} else if (key.equals("ss.runExperiments")){
				runExperiments = value.equals("true") ? true : false;
			} else if (key.equals("ss.visualize")){
				visualize = value.equals("true") ? true : false;
			} else if (key.equals("ss.digger.adaptive")){
				adaptiveDig = value.equals("true") ? true : false;
			} else if (key.equals("ss.digger.digStartInterval")){
				digStart = Integer.valueOf(value);
			} else if (key.equals("ss.digMaxInterval")){
				digMaxInterval = Integer.valueOf(value);
			} else if (key.equals("ss.digIncrement")){
				digIncrement = Integer.valueOf(value);
			} else if (key.equals("ss.sampleInterval")){
				sampleInterval = Integer.valueOf(value);
//				RSTPacketStats.INTERVAL = sampleInterval*1000;
//				Main.getInstance().updateRSTTimer();
			} else if (key.equals("ss.override")){
				override= value.equals("true") ? true : false;
			}  else if (key.equals("ss.alarmThresh")){
				alarmThreshold = Integer.valueOf(value);
			} else if (key.equals("ss.anomCountLocal")){
				anomCountThreshold = Integer.valueOf(value);
			} else if (key.equals("ss.pctDiffTrigger")){
				pctDiffTrigger = Integer.valueOf(value);
			} else if (key.equals("ss.suppressWarnings")){
				suppressWarnings = value.equals("true") ? true : false;
			} else if (key.equals("ss.globalOverride")){
				globalOverride = value.equals("true") ? true : false;
			} else if (key.equals("ss.privacyUpdateInterval")){
				privacyUpdateInterval = Integer.valueOf(value);
			} else if (key.equals("ss.torrentDownloadInterval")){
				torrentDownloadInterval = Integer.valueOf(value);
			} else if (key.equals("ss.privacyLevel")){
				privacyLevel = Integer.valueOf(value);
			} else if (key.startsWith("ss.torrentURL")){
				tempURLs.add(value);
			} else if (key.startsWith("ss.torrentRotateInterval")){
				torrentRotateInterval = Integer.valueOf(value);
			} else if (key.startsWith("ss.enabled")){
				enabled = value.equals("true") ? true : false;
			}else if (key.startsWith("ss.torrentDataDir")){
				torrentDataDir = value;
			}else if (key.startsWith("ss.maxFileSize")){
				maxFileSize = Integer.valueOf(value);
			}else {
				if (DEBUG) System.err.println("Unknown key: "+key);
			}
			} catch (Exception e){
				System.err.println("Error processing property...");
				e.printStackTrace();
			} catch (Error e){
				System.err.println("Error processing property...");
				e.printStackTrace();
			}			
		
		} // end for
		
		// if no uuid, generate it now
		if (uuid==null){
			uuid = UUID.randomUUID().toString();
			props.put("ss.uuid", uuid);
			
			writeProperties(props, Main.getPropertiesLocations());
		}
		
		if (tempURLs.size()>0) torrentURLs = tempURLs.toArray(new String[]{});
		
	}


	public int getDbReconnectInterval() {
		// TODO Auto-generated method stub
		return dbReconnectInterval;
	}


	public void setDbReconnectInterval(int dbReconnectInterval) {
		this.dbReconnectInterval = dbReconnectInterval;
	}


	public void setReportingTimeout(int timeout) {
		this.reportingTimeout = timeout;
		
	}


	public int getReportingTimeout() {
		return reportingTimeout;
	}


	public void setMaxPings(int value) {
		this.maxPings = value;
		
	}
	
	public int getMaxPings(){
		return maxPings;
	}


	public String getUUID() {
		return uuid;
	}
	
	public void setUUID(String uuid){
		this.uuid = uuid;
	}


	public boolean isStreamDebug() {
		// TODO Auto-generated method stub
		return this.streamDebug;
	}
	
	public void setStreamDebug(boolean streamDebug){
		this.streamDebug = streamDebug;
	}


	public boolean isRunExperiments() {
		// TODO Auto-generated method stub
		return runExperiments;
	}
	
	public void setRunExperiments(boolean runExperiments){
		this.runExperiments = runExperiments;
	}


	public boolean isVisualize() {
		// TODO Auto-generated method stub
		return visualize;
	}


	public boolean isAdaptiveDig() {
		return adaptiveDig;
	}


	public void setAdaptiveDig(boolean adaptiveDig) {
		this.adaptiveDig = adaptiveDig;
	}


	public long getDigStart() {
		return digStart;
	}


	public void setDigStart(long digStart) {
		this.digStart = digStart;
	}


	public long getDigMaxInterval() {
		return digMaxInterval;
	}


	public void setDigMaxInterval(long digMaxInterval) {
		this.digMaxInterval = digMaxInterval;
	}


	public long getDigIncrement() {
		return digIncrement;
	}


	public void setDigIncrement(long digIncrement) {
		this.digIncrement = digIncrement;
	}


	public void setDropVivaldiPct(int dropVivaldiPct) {
		this.dropVivaldiPct = dropVivaldiPct;
		
	}
	
	public int getDropVivaldiPct(){
		return dropVivaldiPct;
	}


	public int getSampleInterval() {
		return sampleInterval;
	}


	public void setSampleInterval(int sampleInterval) {
		this.sampleInterval = sampleInterval;
	}


	public void setOverride(boolean override) {
		this.override = override;
		
	}
	
	public boolean isOverride(){ return override;}


	public int getAlarmThreshold() {
		return alarmThreshold ;
	}
	
	public void setAlarmThreshold(int threshold){
		this.alarmThreshold = threshold;
	}


	public int getAnomCountThreshold() {
		return this.anomCountThreshold;
	}
	
	public void setAnomCountThreshold(int thresh){
		this.anomCountThreshold = thresh;
	}
	
	public void setOnoRunning(boolean onoRunning){
		this.onoRunning = onoRunning;
	}
	
	public boolean isOnoRunning(){
		return onoRunning;
	}


	public Integer getPctDiffTrigger() {
		return pctDiffTrigger;
	}


	public void setPctDiffTrigger(Integer pctDiffTrigger) {
		this.pctDiffTrigger = pctDiffTrigger;
	}



	public boolean isSuppressWarnings() {
		return suppressWarnings;
	}


	public void setSuppressWarnings(boolean suppressWarnings) {
		this.suppressWarnings = suppressWarnings;
	}


	public boolean isGlobalOverride() {
		return globalOverride;
	}


	public void setGlobalOverride(boolean globalOverride) {
		this.globalOverride = globalOverride;
	}


	public int getPrivacyUpdateInterval() {
		return privacyUpdateInterval;
	}


	public int getTorrentDownloadInterval() {		
		return torrentDownloadInterval;
	}


	public void setPrivacyLevel(int privacyLevel) {
		this.privacyLevel = privacyLevel;
		
	}
	
	public int getPrivacyLevel(){
		return privacyLevel;
	}


	public void setTorrentURLs(String[] torrentURLs) {
		this.torrentURLs  = torrentURLs;
		
	}
	
	public String[] getTorrentURLs(){
		return torrentURLs;
	}


	public long getTorrentRotateInterval() {
		// TODO Auto-generated method stub
		return torrentRotateInterval ;
	}
	
	public void setTorrentRotateInterval(long torrentRotateInterval){
		this.torrentRotateInterval = torrentRotateInterval;
	}


	public boolean isEnabled() {
		// TODO Auto-generated method stub
		return enabled ;
	}
	
	public void setEnabled(boolean enabled){
		this.enabled = enabled;
	}


	public String getTorrentDataDir() {
		return torrentDataDir;
	}


	public void setTorrentDataDir(String torrentDataDir) {
		this.torrentDataDir = torrentDataDir;
	}


	public int getStatsUpdateTime() {
		return statsUpdateTime;
	}


	public void setStatsUpdateTime(int statsUpdateTime) {
		this.statsUpdateTime = statsUpdateTime;
	}


	public int getMaxFileSize() {		
		return maxFileSize;
	}


	public void setMaxFileSize(int maxFileSize) {
		this.maxFileSize = maxFileSize;
	}

}
