/**
 * SwarmScreen Project
 *
 * File:         Driver.java
 * RCS:          $Id: Driver.java,v 1.5 2009/03/09 11:51:53 drc915 Exp $
 * Description:  Driver class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 28, 2009 at 3:52:39 PM
 * Language:     Java
 * Package:      edu.northwestern.swarmscreen
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2009, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.swarmscreen.analysis;

import java.io.File;

import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;

import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadException;
import org.gudy.azureus2.plugins.download.DownloadRemovalVetoException;
import org.gudy.azureus2.plugins.logging.LoggerChannel;
import org.gudy.azureus2.plugins.torrent.Torrent;
import org.gudy.azureus2.plugins.torrent.TorrentEncodingException;
import org.gudy.azureus2.plugins.torrent.TorrentException;
import org.gudy.azureus2.plugins.utils.UTTimerEvent;
import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderException;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderFactory;

import edu.northwestern.swarmscreen.Main;
import edu.northwestern.swarmscreen.SSConfiguration;
import edu.northwestern.swarmscreen.connection.ConnectionProfiler;
import edu.northwestern.swarmscreen.torrents.TorrentManager;
import edu.northwestern.swarmscreen.torrents.TorrentManager.DownloadStats;

/**
 * @author David Choffnes
 *
 */
public class Driver implements UTTimerEventPerformer{

	private static Driver self;
	
	enum TestType {
		FIXED_DOWNLOADS,
		CHANGE_SPF,
		VARIABLE_DOWNLOADS,
		UNTETHER
	}
	
	StringBuffer outputStats = new StringBuffer();
	boolean firstRun = true;
	
	TestType tt = TestType.FIXED_DOWNLOADS;
	int fixedSPF = 100;
	int fixedDuration = 30; // minutes
	String tempDir = "C:\\projects\\swarmscreen";
	private ArrayList<URL> torrentURLs;
	private String dataDir = "g:\\swarmscreen\\";
	
	public Driver(){
		Main.getInstance().getTimer("Driver").addEvent(15*1000, this);
		
		torrentURLs = new ArrayList<URL>();
		try {
//			torrentURLs.add(new URL("http://torrents.thepiratebay.org/4744575/Battlestar.Galactica.S04E17.HDTV.XviD-0TV.avi.4744575.TPB.torrent"));
//			torrentURLs.add(new URL("http://torrents.thepiratebay.org/4738979/Heroes.S03E17.HDTV.XviD-LOL.avi.4738979.TPB.torrent"));
//			torrentURLs.add(new URL("http://torrents.thepiratebay.org/4742403/Lost.S05E07.HDTV.XviD-XOR.avi.4742403.TPB.torrent"));
			torrentURLs.add(new URL("http://www.tuxdistro.com/download.php?id=1373&name=Fedora-10-i386-DVD.torrent"));
			torrentURLs.add(new URL("http://www.tuxdistro.com/download.php?id=1418&name=openSUSE-11.1-DVD-i586.iso.torrent"));
			torrentURLs.add(new URL("http://www.tuxdistro.com/download.php?id=1520&name=dreamlinux-3.5-final.4745973.TPB.torrent"));
//			torrentURLs.add(new URL("http://download.opensuse.org/distribution/openSUSE-stable/iso/openSUSE-11.1-DVD-i586.iso.torrent"));
			
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	public static Driver getInstance(){
		if (self == null){
			self = new Driver();			
		}
		return self;
	}
	
	public void dumpStats(){
		File f = new File(tempDir);
		if (!f.exists()) f.mkdirs();
		String fname = tempDir+File.separator+tt.toString()+"-"+System.currentTimeMillis();
		f = new File(fname);
		FileWriter fw;		
		if (f.exists()) f.delete();
		try {
			
			fw = new FileWriter( fname, true );
			
			
			fw.write(outputStats.toString());
			fw.flush();
			fw.close();
		} catch (IOException e) {
			
			e.printStackTrace();
		}
	}

	
	public void perform(UTTimerEvent event) {
		if (firstRun){			
			for (Download d : Main.getPluginInterface().getDownloadManager().getDownloads()){
				try {
					d.stopDownload();
					d.remove(true, true);
				} catch (DownloadException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (DownloadRemovalVetoException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			firstRun = false;
			
			// set up tests
			if (tt.equals(TestType.CHANGE_SPF)){
				setupSPF();
			} else if (tt.equals(TestType.FIXED_DOWNLOADS)){
				setupFixedDls();
			} else if (tt.equals(TestType.VARIABLE_DOWNLOADS)){
				setupVariableDls();
			} else if (tt.equals(TestType.UNTETHER)){
				setupUntether();
			}
			
			Main.getInstance().getTimer("Driver").addPeriodicEvent(5*1000, this);			
		} else  
			grabStats();
//		
//		if (tt.equals(TestType.CHANGE_SPF)){
//			
//		} else if (tt.equals(TestType.FIXED_DOWNLOADS)){
//			
//		} else if (tt.equals(TestType.VARIABLE_DOWNLOADS)){
//			
//		}
		
	}

	private void setupUntether() {
		int i = 0;
		for (URL u : torrentURLs){
			if (i>=2) break; 
			addTorrent(u);
			i++;
		}
		SSConfiguration.getInstance().setPrivacyLevel(fixedSPF);
		
		Main.getInstance().getTimer("Driver").addEvent(System.currentTimeMillis()+fixedDuration*60*1000L, new UTTimerEventPerformer(){

			
			public void perform(UTTimerEvent event) {
				dumpStats();
				Main.getLogger().logAlert(LoggerChannel.LT_INFORMATION, "Test complete!");
				SSConfiguration.getInstance().setEnabled(true);
			}});
		
		Main.getInstance().getTimer("Driver").addEvent(System.currentTimeMillis()+(fixedDuration/4)*60*1000L, new UTTimerEventPerformer(){

			
			public void perform(UTTimerEvent event) {
				SSConfiguration.getInstance().setEnabled(false);
				
			}});
		
	}

	private void setupVariableDls() {
		URL u = torrentURLs.remove(0);
		addTorrent(u);
		
		SSConfiguration.getInstance().setPrivacyLevel(100);
		
		for (long t : new long[]{10*60*1000L, 20*60*1000L}){
			
			Main.getInstance().getTimer("Driver").addEvent(System.currentTimeMillis()+t, new UTTimerEventPerformer(){

				
				public void perform(UTTimerEvent event) {
					URL u = torrentURLs.remove(0);
					addTorrent(u);
					
				}});
		}
		

		Main.getInstance().getTimer("Driver").addEvent(System.currentTimeMillis()+30*60*1000L, new UTTimerEventPerformer(){

			
			public void perform(UTTimerEvent event) {
				dumpStats();
				Main.getLogger().logAlert(LoggerChannel.LT_INFORMATION, "Test complete!");
				
			}});
		
	}

	private void setupFixedDls() {
		int i = 0;
		for (URL u : torrentURLs){
			if (i>=2) break; 
			addTorrent(u);
			i++;
		}
		SSConfiguration.getInstance().setPrivacyLevel(fixedSPF);
		
		Main.getInstance().getTimer("Driver").addEvent(System.currentTimeMillis()+fixedDuration*60*1000L, new UTTimerEventPerformer(){

			
			public void perform(UTTimerEvent event) {
				dumpStats();
				Main.getLogger().logAlert(LoggerChannel.LT_INFORMATION, "Test complete!");
				
			}});
		
Main.getInstance().getTimer("Driver").addEvent(System.currentTimeMillis()+(1+fixedDuration)*60*1000L, new UTTimerEventPerformer(){

			
			public void perform(UTTimerEvent event) {
				TorrentManager.getInstance().deleteTransientTorrents();
				
			}});
	}

	private void setupSPF() {
		int j = 0;
		for (URL u : torrentURLs){
			if (j>=2) break; 
			addTorrent(u);
			j++;
		}
		
		// set up events for changing SPF
		long timeAt = System.currentTimeMillis()+5*60*1000;
		long interval = 5*60*1000L;
		SSConfiguration.getInstance().setPrivacyLevel(50);
		for (int i = 60; i <=100; i+=10){
			final int spf = i;
			System.out.println("Setting SPF to "+spf+" at: "+DateFormat.getDateTimeInstance().format(
					new Date(timeAt)));
			Main.getInstance().getTimer("Driver").addEvent(timeAt+interval, new UTTimerEventPerformer(){

				
				public void perform(UTTimerEvent event) {
					SSConfiguration.getInstance().setPrivacyLevel(spf);
					
				}});
			timeAt+=interval;
		}
		
		for (int i = 90; i >0; i-=10){
			final int spf = i;
			System.out.println("Setting SPF to "+spf+" at: "+DateFormat.getDateTimeInstance().format(
					new Date(timeAt)));
			Main.getInstance().getTimer("Driver").addEvent(timeAt+interval, new UTTimerEventPerformer(){

				
				public void perform(UTTimerEvent event) {
					SSConfiguration.getInstance().setPrivacyLevel(spf);
					
				}});
			timeAt+=interval;
		}
		
		System.out.println("Will finish at: "+DateFormat.getDateTimeInstance().format(
				new Date(timeAt)));
		Main.getInstance().getTimer("Driver").addEvent(timeAt+interval, new UTTimerEventPerformer(){

			
			public void perform(UTTimerEvent event) {
				dumpStats();
				Main.getLogger().logAlert(LoggerChannel.LT_INFORMATION, "Test complete!");
				
			}});
	}

	private void grabStats() {
		long time = System.currentTimeMillis();
		System.out.println("Grabbing stats @"+DateFormat.getDateTimeInstance().format(
				new Date(time)));
		double ratio = ConnectionProfiler.getInstance().getConnectionTimesRatio();
		double sim = ConnectionProfiler.getInstance().getConnectionDistributionSimilarity();
		DownloadStats ds = TorrentManager.getInstance().getDownloadStats();
		int spf = SSConfiguration.getInstance().getPrivacyLevel();
		outputStats.append(time+"\t"+ratio+"\t"+sim+"\t"+
				ds.toString()+"\t"+spf+"\n");
		
	}
	
	public void addTorrent(URL u){
		Torrent t = null;
		ResourceDownloaderFactory factory = Main.getPluginInterface().getUtilities().getResourceDownloaderFactory();
		ResourceDownloader rd = factory.create(u);				
		ResourceDownloader rdt = factory.getTimeoutDownloader(rd, 120*1000);
		try {
			t = Main.getPluginInterface().getTorrentManager().createFromBEncodedInputStream(rdt.download());	
			t.setDefaultEncoding();
		} catch (ResourceDownloaderException e) {
			e.printStackTrace();			
			
		} catch (TorrentEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TorrentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			Download d = Main.getPluginInterface().getDownloadManager().addDownload(t, 
					null, 
					new File(dataDir ));
		} catch (DownloadException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
	
}
