/**
 * SwarmScreen Project
 *
 * File:         GenerateGraphs.java
 * RCS:          $Id: GenerateGraphs.java,v 1.7 2009/03/10 03:12:33 drc915 Exp $
 * Description:  GenerateGraphs class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Mar 1, 2009 at 5:28:28 PM
 * Language:     Java
 * Package:      edu.northwestern.swarmscreen.analysis
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2009, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.swarmscreen.analysis;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.Map.Entry;



/**
 * @author David Choffnes
 *
 */
public class GenerateGraphs {
	
	static String baseDir = "C:\\papers\\2009\\sosp-hiding-crowd\\data\\";
	static String outputBaseDir = "C:\\projects\\swarmscreen\\";
	private static int pointSize = 4;

	
	public static void main(String args[]){
		
		doUntether();
		
		doVarySPF2();
		
		doVarySPF();
		
		doFixed();
		
		doVaryDownloads();
		
		doBandwidthRatio();

		
	} // end main
	
	private static void doUntether() {
		String files[] = new String[]{"UNTETHER-1236309714156",
							"FIXED_DOWNLOADS-1236313773359"};

		StringBuffer output = new StringBuffer();

		ArrayList<Long> lines = new ArrayList<Long>();
	    String outputDir = baseDir+File.separator+"graphs"+File.separator;
	    File f = new File(outputDir);
	    if (!f.exists())f.mkdirs();
	       double maxTime = -1;
	       double minRatio = Double.MAX_VALUE; 
	       double maxRatio = Double.MIN_VALUE;
	       ArrayList<String> namesVar = new ArrayList<String>();
	       
		for (String file : files){


			long firstTs = -1;
			int lastSpf = -1;
		try {
	        BufferedReader  in = new BufferedReader(
	        		new FileReader(baseDir+file));
	        String line;				        
	       
	       
	        
	        int i=0;
	        while ((line = in.readLine()) != null) {
	        	if (line.length()>0){
	        		String cols[] = line.split("\t");
	        		if (firstTs < 0){
	    				firstTs = Long.parseLong(cols[0]);
	    				lastSpf = Integer.parseInt(cols[10]);
	    			}
	        		if (lastSpf != Integer.parseInt(cols[10])){
//	    				lines.add(Long.parseLong(cols[0]) - firstTs);
//	    				lastSpf = Integer.parseInt(cols[10]);
	    			}
	        		maxTime = ((Long.parseLong(cols[0]) - firstTs)/60000.0);
	        		output.append(maxTime+"\t" +
	        				cols[1] + "\t" + cols[2]+"\n");
	        		
	        		double ratio = Double.parseDouble(cols[1]);
	        		if (ratio<minRatio) minRatio=ratio;
	        		if (ratio>maxRatio) maxRatio =ratio;
	        	}
	        }
	        in.close();
	  
	        
	    } catch (IOException e) {
	    	e.printStackTrace();	    	
	    }

	    // output file

	    String outputFile = outputDir+file+".dat";
	    writeToFile(outputFile, output.toString());
		
	    
//	    StringBuffer arrowBuffer = new StringBuffer();
//	    for (Long t : lines ){
//	    	double bucket = t/60000.0;
//	    	arrowBuffer.append( "set arrow from "+bucket+","+0+" to " +
//				bucket+", "+2+" nohead lt 0 lw 1\n");
//	    }
	    

//	    
//	    arrowText+="set object 2 rect from 0,0.8 to "+maxTime+",1.2 fc rgb \"#D3D3D3\" norborder\n";
//	    
//	    // p-value limits
//	    arrowText += "set arrow from "+0+","+0.5+" to " +
//			maxTime+", "+0.5+" nohead lt 3 lw 1\n";
//	    arrowText += "set arrow from "+0+","+0.1+" to " +
//		maxTime+", "+0.1+" nohead lt 3 lw 1\n";
	    String label;
	   if (outputFile.toLowerCase().contains("untether")){
		   label = "Disabled at 15'";
		   namesVar.add(
					"\""+outputFile+"\" usi 1:2 axis x1y1 title \""+label+"\" with lines lt 1 lw "+pointSize +", \\\n");
	   } else {
		   label = "Enabled";
		   namesVar.add(
					"\""+outputFile+"\" usi 1:2 axis x1y1 title \""+label+"\" with lines lt 4 lw "+pointSize +", \\\n");

	   }
	   //		namesVar.add("\""+outputFile+"\" usi 1:3 axis x1y2 title \"P-value\" with lines lw "+pointSize+", \\\n");
		}
		
	    String arrowText = "";
//	    arrowText += "set arrow from "+0+","+0.8+" to " +
//				maxTime+", "+0.8+" nohead lt 0 lw 1\n";
//	    arrowText += "set arrow from "+0+","+1.2+" to " +
//		maxTime+", "+1.2+" nohead lt 0 lw 1\n";
	    arrowText += "set arrow from "+15+","+0+" to " +
		15+", "+2+" nohead lt 0 lw 1\n";
	    arrowText+="set style rectangle fc rgb \"#D3D3D3\" fs solid noborder\n";
	    arrowText+="set object 2 rect from 0,0.8 to "+
	    maxTime+",1.2\n";// fc rgb \"#D3D3D3\" border 0 \n";

		String file = "UntetherCompare";
		GnuPlotter gp = new GnuPlotter();
		gp.setYLabel("Ratio");	
		//gp.setY2Label("P-value");	
		gp.setXLabel("Time (min)");
		gp.setTitle("");
		gp.setPlotFiles(namesVar);
		gp.setOutputFile(outputDir+file);
		gp.setLogX(false);
		gp.setLogY(false);
		gp.setYTics("");
		gp.setYRange("[0:2]");
		//gp.setY2Range("[0:0.2]");
		gp.setXRange("[0:"+maxTime+"]");
		gp.setOtherText(arrowText);
		gp.setKeyText("right bottom");
		gp.generatePlot();
		
	}

	private static void doBandwidthRatio() {
		
		HashMap<String, String> files = new HashMap<String, String>();
//		files.put("100", "FIXED_DOWNLOADS-1236102864359");
//		files.put("50", "FIXED_DOWNLOADS-1236174669625");
		boolean DOWNLOAD_ONLY = false;
//		files.put("10", "FIXED_DOWNLOADS-1236229903953");
//		files.put("50", "FIXED_DOWNLOADS-1236226137109");
//		files.put("100", "FIXED_DOWNLOADS-1236220503765");
		
		files.put("10", "FIXED_DOWNLOADS-1236385930375");
		files.put("50", "FIXED_DOWNLOADS-1236387822390");
		files.put("100", "FIXED_DOWNLOADS-1236391829531");
		
	    String outputDir = outputBaseDir+File.separator+"graphs"+File.separator;
	    File f = new File(outputDir);
	    if (!f.exists())f.mkdirs();
	    
		StringBuffer output = new StringBuffer();
		StringBuffer output2 = new StringBuffer();
		long firstTs = -1;
		int lastSpf = -1;
		ArrayList<Long> lines = new ArrayList<Long>();
		TreeMap<String, ArrayList<Double>> ratios = new TreeMap<String, ArrayList<Double>>();
		
	       double minRatio = Double.MAX_VALUE; 
	       double maxRatio = Double.MIN_VALUE;
	       double maxTime = -1;
	       
	       for (Entry<String, String> ent : files.entrySet()){
	    	   String file = ent.getValue();
	    	   ratios.put("Upload-"+ent.getKey(),new ArrayList<Double>());
	   		ratios.put("Download-"+ent.getKey(),new ArrayList<Double>());
		try {
	        BufferedReader  in = new BufferedReader(
	        		new FileReader(baseDir+file));
	        String line;				        
	       
	       
	        
	        int i=0;
	        while ((line = in.readLine()) != null) {
	        	if (line.length()>0){
	        		String cols[] = line.split("\t");
	        		if (firstTs < 0){
	    				firstTs = Long.parseLong(cols[0]);
	    				lastSpf = Integer.parseInt(cols[10]);
	    			}
	        		if (lastSpf != Integer.parseInt(cols[10])){
//	    				lines.add(Long.parseLong(cols[0]) - firstTs);
//	    				lastSpf = Integer.parseInt(cols[10]);
	    			}
	        		maxTime = ((Long.parseLong(cols[0]) - firstTs)/60000.0);
	        		int dlUser = Integer.parseInt(cols[5]);
	        		int dlT = Integer.parseInt(cols[6]);
	        		int ulUser = Integer.parseInt(cols[7]);
	        		int ulT = Integer.parseInt(cols[8]);
	        		if (dlUser>0 && dlT>0){
	        			output.append(maxTime+"\t" +
	        				(dlT/((double)dlUser))+"\n");
	        			ratios.get("Download-"+ent.getKey()).add((dlT/((double)dlUser)));
	        		}
	        		if (ulUser>0 && ulT>0){
	        			output2.append(maxTime+"\t" +
	        				(ulT/((double)ulUser))+"\n");
	        			ratios.get("Upload-"+ent.getKey()).add((ulT/((double)ulUser)));
	        		}
	        		
	        		
	        	}
	        }
	        in.close();
	  
	        
	    } catch (IOException e) {
	    	e.printStackTrace();	    	
	    }
	       

	    // output file
	    String outputFile = outputBaseDir+"DownloadRatio-"+file+".dat";
	    writeToFile(outputFile, output.toString());
	    String outputFile2 = outputBaseDir+"UploadRatio-"+file+".dat";
	    writeToFile(outputFile2, output2.toString());
	    
//	    StringBuffer arrowBuffer = new StringBuffer();
//	    for (Long t : lines ){
//	    	double bucket = t/60000.0;
//	    	arrowBuffer.append( "set arrow from "+bucket+","+0+" to " +
//				bucket+", "+2+" nohead lt 0 lw 1\n");
//	    }
	    
	    String arrowText = "";
//	    arrowText += "set arrow from "+0+","+0.8+" to " +
//				maxTime+", "+0.8+" nohead lt 0 lw 1\n";
//	    arrowText += "set arrow from "+0+","+1.2+" to " +
//		maxTime+", "+1.2+" nohead lt 0 lw 1\n";
//	    
//	    // p-value limits
//	    arrowText += "set arrow from "+0+","+0.5+" to " +
//			maxTime+", "+0.5+" nohead lt 3 lw 1\n";
//	    arrowText += "set arrow from "+0+","+0.1+" to " +
//		maxTime+", "+0.1+" nohead lt 3 lw 1\n";
	    
	    ArrayList<String> namesVar = new ArrayList<String>();
	    namesVar.add(
				"\""+outputFile+"\" usi 1:2 axis x1y1 title \"Download\" with points pt 8 ps "+pointSize +", \\\n");
		if (!DOWNLOAD_ONLY) namesVar.add("\""+outputFile2+"\" usi 1:2 axis x1y1 title \"Upload\" with points pt 6 ps "+pointSize+", \\\n");
		
		GnuPlotter gp = new GnuPlotter();
		gp.setYLabel("Ratio (User:Transient)");	
//		gp.setY2Label("Ratio (User/Transient)");	
		gp.setXLabel("Time (min)");
		gp.setTitle("");
		gp.setPlotFiles(namesVar);
		gp.setOutputFile(outputDir+"Ratios"+file);
		gp.setLogX(false);
		gp.setLogY(false);
		gp.setYTics("");
		gp.setYRange("[0:1]");
		//gp.setY2Range("[0:1]");
		gp.setXRange("[0:"+maxTime+"]");
		//gp.setOtherText("set y2tics\nset ytics nomirror\n"+arrowText);
		gp.setKeyText("out horiz center top");
		gp.generatePlot();
	       }
	       
		ArrayList<String> plotFiles = new ArrayList<String>();
		ArrayList<Pair<Integer, Integer>> curveTypes = getCurveTypes();
		String dir = outputDir;
		f = new File(dir+"data"+File.separator);
		if (!f.exists()) f.mkdirs();
//			for (int size : netSizes) {		
		for (Entry<String, ArrayList<Double>> ent : ratios.entrySet()){
			if (DOWNLOAD_ONLY && ent.getKey().contains("Download")) continue; 
			String fname= dir +"data"+File.separator+"Ratio-"+ent.getKey();
			generateCDF(plotFiles, ent, fname, 
				ent.getKey(), 
				curveTypes);
		}
			
		String otherText = "set arrow from "+1+","+0+" to " +
		1+", "+1+" nohead lt 3 lw 1\n";
		otherText += "set arrow from "+0.5+","+0+" to " +
		0.5+", "+1+" nohead lt 3 lw 1\n";
		otherText += "set arrow from "+0.1+","+0+" to " +
		0.1+", "+1+" nohead lt 3 lw 1\n";
		
		GnuPlotter gp = new GnuPlotter();
		gp.setYLabel("CDF [X<r]");
		gp.setXLabel("R_b (Bandwidth ratio)");
		gp.setTitle("");
		gp.setPlotFiles(plotFiles);
		gp.setOutputFile(dir+"RatioCDF");
//		gp.setLogX(true);
//			gp.setLogY(true);
//			gp.setYTics(yTics);
//			gp.setYRange(yRange);
		gp.setXRange("[0:2]");
		gp.setOtherText(otherText);
		gp.setKeyText("bottom right");
		gp.generatePlot();
		
	}

	private static void doFixed() {
		String file = "FIXED_DOWNLOADS-1236102864359";

		StringBuffer output = new StringBuffer();
		long firstTs = -1;
		int lastSpf = -1;
		ArrayList<Long> lines = new ArrayList<Long>();
	       double minRatio = Double.MAX_VALUE; 
	       double maxRatio = Double.MIN_VALUE;
	       double maxTime = -1;
		try {
	        BufferedReader  in = new BufferedReader(
	        		new FileReader(baseDir+file));
	        String line;				        
	       
	       
	        
	        int i=0;
	        while ((line = in.readLine()) != null) {
	        	if (line.length()>0){
	        		String cols[] = line.split("\t");
	        		if (firstTs < 0){
	    				firstTs = Long.parseLong(cols[0]);
	    				lastSpf = Integer.parseInt(cols[10]);
	    			}
	        		if (lastSpf != Integer.parseInt(cols[10])){
//	    				lines.add(Long.parseLong(cols[0]) - firstTs);
//	    				lastSpf = Integer.parseInt(cols[10]);
	    			}
	        		maxTime = ((Long.parseLong(cols[0]) - firstTs)/60000.0);
	        		output.append(maxTime+"\t" +
	        				cols[1] + "\t" + cols[2]+"\n");
	        		
	        		double ratio = Double.parseDouble(cols[1]);
	        		if (ratio<minRatio) minRatio=ratio;
	        		if (ratio>maxRatio) maxRatio =ratio;
	        	}
	        }
	        in.close();
	  
	        
	    } catch (IOException e) {
	    	e.printStackTrace();	    	
	    }

	    // output file
	    String outputDir = outputBaseDir+File.separator+"graphs"+File.separator;
	    File f = new File(outputDir);
	    if (!f.exists())f.mkdirs();
	    String outputFile = outputDir+file+".dat";
	    writeToFile(outputFile, output.toString());
	    
//	    StringBuffer arrowBuffer = new StringBuffer();
//	    for (Long t : lines ){
//	    	double bucket = t/60000.0;
//	    	arrowBuffer.append( "set arrow from "+bucket+","+0+" to " +
//				bucket+", "+2+" nohead lt 0 lw 1\n");
//	    }
	    
	    String arrowText = "";
//	    arrowText += "set arrow from "+0+","+0.8+" to " +
//				maxTime+", "+0.8+" nohead lt 0 lw 1\n";
//	    arrowText += "set arrow from "+0+","+1.2+" to " +
//		maxTime+", "+1.2+" nohead lt 0 lw 1\n";
	    
	    arrowText+="set style rectangle fc rgb \"#D3D3D3\" fs solid noborder\n";
	    arrowText+="set object 2 rect from 0,0.8 to "+
	    maxTime+",1.2\n";// fc rgb \"#D3D3D3\" border 0 \n";
	    
	    // p-value limits
	    arrowText += "set arrow from "+0+","+0.5+" to " +
			maxTime+", "+0.5+" nohead lt 3 lw 1\n";
	    arrowText += "set arrow from "+0+","+0.1+" to " +
		maxTime+", "+0.1+" nohead lt 3 lw 1\n";
	    
	    ArrayList<String> namesVar = new ArrayList<String>();
	    namesVar.add(
				"\""+outputFile+"\" usi 1:2 axis x1y1 title \"Ratio\" with lines lw "+pointSize +", \\\n");
		namesVar.add("\""+outputFile+"\" usi 1:3 axis x1y2 title \"P-value\" with lines lw "+pointSize+", \\\n");
		
		GnuPlotter gp = new GnuPlotter();
		gp.setYLabel("Ratio");	
		gp.setY2Label("P-value");	
		gp.setXLabel("Time (min)");
		gp.setTitle("");
		gp.setPlotFiles(namesVar);
		gp.setOutputFile(outputDir+file);
		gp.setLogX(false);
		gp.setLogY(false);
		gp.setYTics("");
		gp.setYRange("[0:2]");
		gp.setY2Range("[0:0.2]");
		gp.setXRange("[0:"+maxTime+"]");
		gp.setOtherText("set y2tics\nset ytics nomirror\n"+arrowText);
		gp.setKeyText("right top");
		gp.generatePlot();
		
	}
	
	private static void doVarySPF2() {
		
		String files[] = new String[]{"FIXED_DOWNLOADS-1236229903953",
				"FIXED_DOWNLOADS-1236226137109",
				"FIXED_DOWNLOADS-1236220503765",};
		double lowWater = 0.8;
		double highWater = 1.2;
		 String outputDir = outputBaseDir+File.separator+"graphs"+File.separator;
		    File f = new File(outputDir);
		    if (!f.exists())f.mkdirs();
		 ArrayList<String> namesVar = new ArrayList<String>();
		
		
		int lastSpf = -1;
	       double minRatio = Double.MAX_VALUE; 
	       double maxRatio = Double.MIN_VALUE;
	       double maxTime = -1;
	       HashMap<String, Double> pctViolations = new HashMap<String, Double>();
		int j = 0;
	       for (String file : files){
			j++;
	    	   StringBuffer output = new StringBuffer();
			double count = 0;
			int violations = 0;
	       int spf = -1;
		try {
	        BufferedReader  in = new BufferedReader(
	        		new FileReader(baseDir+file));
	        String line;				        
	        
	        maxTime = -1;
	        long firstTs = -1;
	        int i=0;
	        while ((line = in.readLine()) != null) {
	        	if (line.length()>0){
	        		String cols[] = line.split("\t");
	        		if (firstTs < 0){
	    				firstTs = Long.parseLong(cols[0]);
	    				spf = Integer.parseInt(cols[10]);
	    			}
	        		double ratio = Double.parseDouble(cols[1]);
	        		if (ratio<0.1) continue;
	        		if (ratio>highWater || ratio < lowWater) violations++;
	        		count++;
	        		maxTime = ((Long.parseLong(cols[0]) - firstTs)/60000.0);
	        		output.append(maxTime+"\t" +
	        				cols[1] + "\t" + cols[2]+"\n");
	        		
	        		if (ratio<minRatio) minRatio=ratio;
	        		if (ratio>maxRatio) maxRatio =ratio;
	        	}
	        }
	        in.close();
	  
	        
	    } catch (IOException e) {
	    	e.printStackTrace();	    	
	    }

	    pctViolations.put(""+spf, (violations/count));
	    // output file
	   
	    String outputFile = outputDir+file+".dat";
	    writeToFile(outputFile, output.toString());
	    
//	    StringBuffer arrowBuffer = new StringBuffer();
//	    for (Long t : lines ){
//	    	double bucket = t/60000.0;
//	    	arrowBuffer.append( "set arrow from "+bucket+","+0+" to " +
//				bucket+", "+2+" nohead lt 0 lw 1\n");
//	    }
	    
//	    String arrowText = "";
//	    arrowText += "set arrow from "+0+","+0.8+" to " +
//				maxTime+", "+0.8+" nohead lt 0 lw 1\n";
//	    arrowText += "set arrow from "+0+","+1.2+" to " +
//		maxTime+", "+1.2+" nohead lt 0 lw 1\n";
//	    

//	    
//	    // p-value limits
//	    arrowText += "set arrow from "+0+","+0.5+" to " +
//			maxTime+", "+0.5+" nohead lt 3 lw 1\n";
//	    arrowText += "set arrow from "+0+","+0.1+" to " +
//		maxTime+", "+0.1+" nohead lt 3 lw 1\n";
	    
	   
	    namesVar.add(
				"\""+outputFile+"\" usi 1:2 axis x1y1 title \"SPF "+spf+"\" with lines lw "+(j+pointSize) +", \\\n");
//		namesVar.add("\""+outputFile+"\" usi 1:3 axis x1y2 title \"P-value\" with lines lw "+pointSize+", \\\n");
		
		} // end for each file
		
		System.out.println(pctViolations);
		   String arrowText="set style rectangle fc rgb \"#D3D3D3\" fs solid noborder\n";
		    arrowText+="set object 2 rect from 0,0.8 to "+
		    maxTime+",1.2\n";// fc rgb \"#D3D3D3\" border 0 \n";
		    
		GnuPlotter gp = new GnuPlotter();
		gp.setYLabel("Ratio");	
		//gp.setY2Label("P-value");	
		gp.setXLabel("Time (min)");
		gp.setTitle("");
		gp.setPlotFiles(namesVar);
		gp.setOutputFile(outputDir+"VarySPFMultiCurve");
		gp.setLogX(false);
		gp.setLogY(false);
		gp.setYTics("");
		gp.setYRange("[0:2]");
		//gp.setY2Range("[0:0.2]");
		gp.setXRange("[0:"+maxTime+"]");
		gp.setOtherText(arrowText);
		gp.setKeyText("right top");
		gp.generatePlot();
		
	}
	
	private static void doVaryDownloads() {
		String file = "VARIABLE_DOWNLOADS-1236172643484";

		StringBuffer output = new StringBuffer();
		long firstTs = -1;
		int lastSpf = -1;
		ArrayList<Long> lines = new ArrayList<Long>();
	       double minRatio = Double.MAX_VALUE; 
	       double maxRatio = Double.MIN_VALUE;
	       double maxTime = -1;
		try {
	        BufferedReader  in = new BufferedReader(
	        		new FileReader(baseDir+file));
	        String line;				        
	       
	       
	        
	        int i=0;
	        while ((line = in.readLine()) != null) {
	        	if (line.length()>0){
	        		String cols[] = line.split("\t");
	        		if (firstTs < 0){
	    				firstTs = Long.parseLong(cols[0]);
	    				lastSpf = Integer.parseInt(cols[9]);
	    			}
//	        		if (lastSpf < Integer.parseInt(cols[9])){
//	    				lines.add(Long.parseLong(cols[0]) - firstTs);
//	    				lastSpf = Integer.parseInt(cols[9]);
//	    			}
	        		maxTime = ((Long.parseLong(cols[0]) - firstTs)/60000.0);
	        		output.append(maxTime+"\t" +
	        				cols[1] + "\t" + cols[2]+"\n");
	        		
	        		double ratio = Double.parseDouble(cols[1]);
	        		if (ratio<minRatio) minRatio=ratio;
	        		if (ratio>maxRatio) maxRatio =ratio;
	        	}
	        }
	        in.close();
	  
	        
	    } catch (IOException e) {
	    	e.printStackTrace();	    	
	    }

	    // output file
	    String outputDir = outputBaseDir+File.separator+"graphs"+File.separator;
	    File f = new File(outputDir);
	    if (!f.exists())f.mkdirs();
	    String outputFile = outputDir+file+".dat";
	    writeToFile(outputFile, output.toString());
	    
	    StringBuffer arrowBuffer = new StringBuffer();
	    lines.add(10*60*1000L);
	    lines.add(20*60*1000L);
	    for (Long t : lines ){
	    	double bucket = t/60000.0;
	    	arrowBuffer.append( "set arrow from "+bucket+","+0+" to " +
				bucket+", "+2+" nohead lt 0 lw 2\n");
	    }
	    
	    String arrowText = arrowBuffer.toString();
//	    arrowText += "set arrow from "+0+","+0.8+" to " +
//				maxTime+", "+0.8+" nohead lt 0 lw 1\n";
//	    arrowText += "set arrow from "+0+","+1.2+" to " +
//		maxTime+", "+1.2+" nohead lt 0 lw 1\n";
	    arrowText+="set style rectangle fc rgb \"#D3D3D3\" fs solid noborder\n";
	    arrowText+="set object 2 rect from 0,0.8 to "+
	    maxTime+",1.2\n";// fc rgb \"#D3D3D3\" border 0 \n";

	    
	    ArrayList<String> namesVar = new ArrayList<String>();
	    namesVar.add(
				"\""+outputFile+"\" usi 1:2 axis x1y1 notitle \"Ratio\" with lines lw "+pointSize +", \\\n");
		//namesVar.add("\""+outputFile+"\" usi 1:3 axis x1y2 title \"P-value\" with lines lw "+pointSize+", \\\n");
		
		GnuPlotter gp = new GnuPlotter();
		gp.setYLabel("Ratio");	
		//gp.setY2Label("P-value");	
		gp.setXLabel("Time (min)");
		gp.setTitle("");
		gp.setPlotFiles(namesVar);
		gp.setOutputFile(outputDir+file);
		gp.setLogX(false);
		gp.setLogY(false);
		gp.setYTics("");
		gp.setYRange("[0:2]");
		//gp.setY2Range("[0:0.1]");
		gp.setXRange("[0:"+maxTime+"]");
		gp.setOtherText(arrowText);
		gp.setKeyText("right top");
		gp.generatePlot();
		
	}

	public static void doVarySPF(){
		String file = "CHANGE_SPF-1236139426734";

		StringBuffer output = new StringBuffer();
		StringBuffer output2 = new StringBuffer();
		long firstTs = -1;
		int lastSpf = -1;
		TreeMap<Long, Integer> lines = new TreeMap<Long, Integer>();
		lines.put(5*60*1000L, 50);
	       double minRatio = Double.MAX_VALUE; 
	       double maxRatio = Double.MIN_VALUE;
	       double maxTime = -1;
		try {
	        BufferedReader  in = new BufferedReader(
	        		new FileReader(baseDir+file));
	        String line;				        
	       
	       
	        String cols[] = null;
	        int i=0;
	        while ((line = in.readLine()) != null) {
	        	if (line.length()>0){
	        		cols = line.split("\t");
	        		if (firstTs < 0){
	    				firstTs = Long.parseLong(cols[0]);
	    				lastSpf = Integer.parseInt(cols[10]);
	    			}
	        		if (lastSpf != Integer.parseInt(cols[10])){
	    				lines.put(Long.parseLong(cols[0]) - firstTs, lastSpf);
	    				lastSpf = Integer.parseInt(cols[10]);
	    			}
	        		maxTime = ((Long.parseLong(cols[0]) - firstTs)/60000.0);
	        		output.append(maxTime+"\t" +
	        				cols[1] + "\t" + cols[2]+"\n");
	        		
	        		double ratio = Double.parseDouble(cols[1]);
	        		if (ratio<minRatio) minRatio=ratio;
	        		if (ratio>maxRatio) maxRatio =ratio;
	        	}
	        }
	        lines.put(Long.parseLong(cols[0]) - firstTs, lastSpf);
	        in.close();
	  
	        
	    } catch (IOException e) {
	    	e.printStackTrace();	    	
	    }

	    // output file
	    String outputDir = outputBaseDir+File.separator+"graphs"+File.separator;
	    File f = new File(outputDir);
	    if (!f.exists())f.mkdirs();
	    String outputFile = outputDir+file+".dat";
	    writeToFile(outputFile, output.toString());
	    
	    String boxText = "set boxwidth 4.5\n" +
	    		"set style fill solid 0.1 noborder\n"; 
	    		//+ "set bmargin 3";
	    
	    StringBuffer arrowBuffer = new StringBuffer();
	    for (Entry<Long, Integer> ent : lines.entrySet() ){
	    	double bucket = ent.getKey()/60000.0;
	    	//arrowBuffer.append( "set arrow from "+bucket+","+0+" to " +
			//	bucket+", "+2+" nohead lt 0 lw 1\n");
	    	output2.append((bucket-2.5)+"\t"+(ent.getValue()/100.0)+"\n");
	    }
	    String outputFile2 = outputDir+"SPF"+file+".dat";
	    writeToFile(outputFile2, output2.toString());
	    
	    ArrayList<String> namesVar = new ArrayList<String>();
		namesVar.add("\""+outputFile2+"\" usi 1:2 axis x1y1 title \"SPF\" with boxes, \\\n");
	    namesVar.add(
				"\""+outputFile+"\" usi 1:2 axis x1y1 title \"Ratio\" with lines lt 1 lw "+pointSize +", \\\n");
		namesVar.add("\""+outputFile+"\" usi 1:3 axis x1y2 title \"P-value\" with lines lt 2 lw "+pointSize+", \\\n");
		
		GnuPlotter gp = new GnuPlotter();
		gp.setYLabel("Ratio");	
		gp.setY2Label("P-value");	
		gp.setXLabel("Time (min)");
		gp.setTitle("");
		gp.setPlotFiles(namesVar);
		gp.setOutputFile(outputDir+file);
		gp.setLogX(false);
		gp.setLogY(false);
		gp.setYTics("");
		gp.setYRange("[0:2]");
		gp.setY2Range("[0:0.1]");
		gp.setXRange("[0:"+maxTime+"]");
		gp.setOtherText("set y2tics\nset ytics nomirror\n"+arrowBuffer.toString()+boxText);
		gp.setKeyText("right top");
		gp.generatePlot();

	}
	
	static void writeToFile(String fname, String output) {
		FileWriter fw;
		File f = new File(fname);
		if (f.exists()) f.delete();
		try {
			
			fw = new FileWriter( fname, true );
			
			
			fw.write(output);
			fw.flush();
			fw.close();
		} catch (IOException e) {
			
			e.printStackTrace();
		}
		
	}
	
	public static ArrayList<Pair<Integer, Integer>> getCurveTypes() {
		ArrayList<Pair<Integer, Integer>> toReturn = new ArrayList<Pair<Integer, Integer>>();
		toReturn.add(new Pair<Integer, Integer>(1,6));
		toReturn.add(new Pair<Integer, Integer>(1,4));
		toReturn.add(new Pair<Integer, Integer>(1,12));
		toReturn.add(new Pair<Integer, Integer>(2,7));
		toReturn.add(new Pair<Integer, Integer>(2,5));
		toReturn.add(new Pair<Integer, Integer>(2,13));
		toReturn.add(new Pair<Integer, Integer>(2,8));
		toReturn.add(new Pair<Integer, Integer>(3,10));
		toReturn.add(new Pair<Integer, Integer>(8,9));
		toReturn.add(new Pair<Integer, Integer>(9,11));
		
		return toReturn;
	}

	/**
	 * @param plotFiles
	 * @param ent
	 * @param fnameStub
	 * @param curveTypes 
	 */
	public static void generateCDF(ArrayList<String> plotFiles,
			Entry ent, String fnameStub,
			String title,
			ArrayList<Pair<Integer, Integer>> curveTypes) {
		int sampleLimitSize = 50;
		double sampleIncr = 0.05;
		String lineText = "title \""+title+"\" w lines lw 5";
		Pair<Integer, Integer> lineStyle =null;
		
		if (((ArrayList)ent.getValue()).size()==0) return;
		Collections.sort(((ArrayList)ent.getValue()));
		int size = ((ArrayList)ent.getValue()).size();
		if (size>sampleLimitSize){
			if (curveTypes.size()==0) curveTypes = getCurveTypes();
			lineStyle = curveTypes.remove(0);
			lineText = "notitle w lines lt "+lineStyle.getKey()+" lw 5";
		}
		
		StringBuffer sb = new StringBuffer();
		StringBuffer sbSample = new StringBuffer();
		int j = 0;
		String firstPoint = "";
		double nextVal = 0;
		double d;
		File f = new File(fnameStub+".dat");
		if (f.exists()) f.delete();
		for (Object o : ((ArrayList)ent.getValue())){
			if (o instanceof Double) d = (Double)o;
			else if (o instanceof Float) d = (Float)o;
			else if (o instanceof Integer) d = (Integer)o;
			else continue;
			sb.append(d+ "\t"+((double)j)/size+"\n");
			if (j==0) firstPoint = sb.toString();
			if (size>sampleLimitSize && ((double)j)/size >=nextVal){
				sbSample.append(d+ "\t"+((double)j)/size+"\n");
				nextVal+=sampleIncr;
			}
			if (sb.length()>10000000){
				appendToFile(fnameStub+".dat", sb.toString());
				sb = new StringBuffer();
			}
			j++;
		}	
		
		appendToFile(fnameStub+".dat", sb.toString());
		plotFiles.add("\""+fnameStub+".dat\" "+lineText +", \\\n");
		
		if (size>sampleLimitSize){
			writeToFile(fnameStub+"-1p.dat", firstPoint);
			writeToFile(fnameStub+"-sample.dat", sbSample.toString());
			plotFiles.add("\""+fnameStub+"-1p.dat\" title \""+title+"\"" +
					" w lp lt "+lineStyle.getKey()+
					" pt " + lineStyle.getValue() + 
				" ps 2 lw 5, \\\n");
			plotFiles.add("\""+fnameStub+"-sample.dat\" notitle " +
					" w points pt " + lineStyle.getValue() + 
				" ps 2 lw 5, \\\n");
		}
		
	}
	
	static public void appendToFile(String fname, String output 
	) {
FileWriter fw;

try {

		fw = new FileWriter( fname, true );
	
	
	fw.write(output);
	fw.flush();
	fw.close();
} catch (IOException e) {
	
	e.printStackTrace();
}


}

	
}
