package edu.northwestern.swarmscreen.analysis;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;


/**
 * Handles plotting of data files with gnuplot, ported from my PHP 
 * implementation.
 * 
 * @author David
 *
 */
public class GnuPlotter {

	private static final String SCRIPT_EXT;
	private static final String TEMP_DIR;
	static {
		if (System.getProperty("os.name").toUpperCase().contains("WINDOWS")){
			SCRIPT_EXT = ".bat";
			TEMP_DIR = "C:\\temp";
		}
		else{
			SCRIPT_EXT = ".sh";
			TEMP_DIR = "/tmp";
		}
	}
	String xLabel="", yLabel="", y2Label=null, title="", outputFile="", yRange="",
	xRange="", yTics="", keyText="", otherText="";
	boolean useColor=false, logY=false, logX=false;
	List<String> plotFiles;
	private int fontSize=30;
	private String y2Range = null;;

	public void generatePlot()
	{
		String useColorText, logTextY, logTextX, output;

		if (useColor) useColorText = "color";
		else useColorText = "monochrome";
		if (logY) logTextY = "set logscale y";
		else logTextY = "set xzeroaxis";
		if (logX) logTextX = "set logscale x";
		else logTextX = "#";

		output = "set size 1.2, 1 \n";
		if (yRange != "") output += "set yrange "+yRange+"\n";
		if (xRange != "") output += "set xrange "+xRange+"\n";
		if (y2Range!=null && y2Range.length()>0 ) output += "set y2range "+y2Range+"\n";

		output+="set origin 0,0 \n";
		output+="set xlabel \""+xLabel+"\" \n";
		output+="set ylabel \""+yLabel+"\" \n";
		if (y2Label!=null) output+="set y2label \""+y2Label+"\" \n";
		output+=logTextX+" \n";
		output+=logTextY+" \n";
		output+="set ytics "+yTics+" \n";
		output+="set title \""+title+"\" \n";
		output+="set key "+keyText+" \n";
		output+="#set yzeroaxis \n";
		output+=otherText+" \n";
		output+="set term postscript enhanced "+useColorText+" eps \"Helvetica\" "+fontSize+" \n";
		output+="set output \""+outputFile.replace('\\', '/')+".eps\" \n";
		output+=" \n";
		output+="plot  ";

		if (plotFiles==null){
			System.err.println("No plot files specified!");
			return;
		}
		for(int i=0; i < plotFiles.size(); i++){
			if (i != plotFiles.size()-1) output += plotFiles.get(i).replace('\\', '/').replace("/\n", "\\\n");
			else if (plotFiles.get(i).contains(",")){
				output += plotFiles.get(i).replace('\\', '/').substring(0, plotFiles.get(i).lastIndexOf(','));
			}
			else output += plotFiles.get(i);
		}



//		echo $output;

		if (outputFile!=""){
//			write the file
			String name = outputFile + ".plt";

			writeToFile(name, output);

			System.out.print( "Done writing graph file, executing graph for "+name+"...\n");
			File f = new File(outputFile+".eps");
			if (f.exists()) f.delete();
			if (System.getProperty("os.name").toUpperCase().contains("WINDOWS")){
				runCommand("\"c:\\program files\\wgnuplot\\wgnuplot.exe\" "+name);				
			}
			else {
			runCommand("gnuplot "+name);

			}
			f = new File(outputFile+".eps");
			if (f.exists()) runCommand("epstopdf "+outputFile+".eps");

		} // end case data to write
	}

	private static void runCommand(String command) {
		String tmpnam = TEMP_DIR + File.separator + "foo" + SCRIPT_EXT ;
		
		try {
			FileOutputStream fos = new FileOutputStream(tmpnam);
			fos.write(command.getBytes());
			fos.close();
			File f = new File(tmpnam);
			f.setExecutable(true);		
			
			Process p = Runtime.getRuntime().exec(tmpnam);
			try {
				p.waitFor();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			p.destroy();
			
			f.delete();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}



	protected void writeToFile(String fname, String output) {
		FileWriter fw;

		try {

			fw = new FileWriter( fname, false );


			fw.write(output);
			fw.flush();
			fw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public String getXLabel() {
		return xLabel;
	}

	public void setXLabel(String label) {
		xLabel = label;
	}

	public String getYLabel() {
		return yLabel;
	}
	
	public void setY2Label(String label) {
		y2Label = label;
	}

	public void setYLabel(String label) {
		yLabel = label;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getOutputFile() {
		return outputFile;
	}

	public void setOutputFile(String outputFile) {
		this.outputFile = outputFile;
	}

	public String getYRange() {
		return yRange;
	}

	public void setYRange(String range) {
		yRange = range;
	}

	public String getXRange() {
		return xRange;
	}

	public void setXRange(String range) {
		xRange = range;
	}

	public String getYTics() {
		return yTics;
	}

	public void setYTics(String tics) {
		yTics = tics;
	}

	public String getKeyText() {
		return keyText;
	}

	public void setKeyText(String keyText) {
		this.keyText = keyText;
	}

	public String getOtherText() {
		return otherText;
	}

	public void setOtherText(String otherText) {
		this.otherText = otherText;
	}

	public boolean isUseColor() {
		return useColor;
	}

	public void setUseColor(boolean useColor) {
		this.useColor = useColor;
	}

	public boolean isLogY() {
		return logY;
	}

	public void setLogY(boolean logY) {
		this.logY = logY;
	}

	public boolean isLogX() {
		return logX;
	}

	public void setLogX(boolean logX) {
		this.logX = logX;
	}

	public List<String> getPlotFiles() {
		return plotFiles;
	}

	public void setPlotFiles(List<String> plotFiles) {
		this.plotFiles = plotFiles;
	}

	public void setFontSize(int fontSize) {
		this.fontSize = fontSize;
		
	}

	public void setY2Range(String range) {
		y2Range  = range;
		
	}

}