/**
 * SwarmScreen Project
 *
 * File:         LanguageConverter.java
 * RCS:          $Id: LanguageConverter.java,v 1.1 2009/04/16 17:59:17 drc915 Exp $
 * Description:  LanguageConverter class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 14, 2009 at 8:23:31 AM
 * Language:     Java
 * Package:      edu.northwestern.swarmscreen.analysis
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2009, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.swarmscreen.analysis;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Properties;

/**
 * @author David Choffnes
 *
 */
public class LanguageConverter {

	private static final String zeros = "000";
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		File f = new File("c:\\workspace\\swarmscreen\\src\\edu\\northwestern\\swarmscreen\\internat\\");
		File outputDir = new File("C:\\temp\\internat\\");
		if (!outputDir.exists()) outputDir.mkdirs();
		
		
		for (File pFile : f.listFiles()){
			if (pFile.isDirectory()) continue;
			Properties p = new Properties();
			StringBuffer sb = new StringBuffer();
			try {
				String input = readFileAsString(pFile);
//				
//				Properties pNew = new Properties();
//				for (Object key : p.keySet()){
//					String s = (String)p.get(key);
////					String out = "";
////					byte[] unicode = s.getBytes("UTF-8");
////					String foo = new String(unicode, "Unicode");
//					sb.append(key+"="+ toEscape(s)+"\n");
//				}
				FileWriter fw = new FileWriter(outputDir+File.separator+pFile.getName());
				fw.write(toEscape(input).replace("\\n", "\n"));
				fw.close();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}
	
	  public static String toEscape(String str) {
		    
		    StringBuffer sb = new StringBuffer();
		    char[] charArr = str.toCharArray();
		    for (int i = 0; i < charArr.length; i++) {
		      if (charArr[i] == '\n')
		        sb.append( "\\n" );
		      else if (charArr[i] == '\t')
		        sb.append( "\\t" );
		      else if (charArr[i] >= 0 && charArr[i] < 128 && charArr[i] != '\\' ) 
		        sb.append( charArr[i] );
		      else
		        sb.append(toEscape(charArr[i]));
		    }
		    return sb.toString();
		  }
		 
		  public static String toEscape(char c) {
		    int n = (int)c;
		    String body = Integer.toHexString(n);
		    return ('\\'+"u" + zeros.substring(0, 4 - body.length()) + body);
		  }

		  private static String readFileAsString(File filePath)
		    throws java.io.IOException{
		      String charset = "ISO8859_1";
		      if (filePath.getName().contains("CN")){
		    	  charset = "UTF-8";
		      } else if (filePath.getName().contains("PL"))  charset = "UTF-8";
		      else if (filePath.getName().contains("RU")) charset = "Cp1251";
		      else if (filePath.getName().contains("SK")) charset = "UTF-8";
			  
			  StringBuffer fileData = new StringBuffer(1000);
		        BufferedInputStream reader = new BufferedInputStream(new FileInputStream(filePath));
		        byte[] buf = new byte[1024];
		        int numRead=0;
		        while((numRead=reader.read(buf)) != -1){
		            if (numRead < buf.length){
		            	byte[] temp = Arrays.copyOfRange(buf, 0, numRead);
		            	buf = temp;
		            }
		        	String s = new String(buf, charset);
		        	fileData.append(s);
		        }
		        reader.close();
		        return fileData.toString();
		    }

		  
}
