/**
 * SwarmScreen Project
 *
 * File:         TorrentFetcher.java
 * RCS:          $Id: TorrentFetcher.java,v 1.8 2009/03/03 21:39:29 drc915 Exp $
 * Description:  TorrentFetcher class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 21, 2009 at 11:09:37 AM
 * Language:     Java
 * Package:      edu.northwestern.swarmscreen.torrents
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2009, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.swarmscreen.torrents;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLDecoder;
import java.nio.charset.Charset;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.Map.Entry;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import org.gudy.azureus2.plugins.logging.LoggerChannel;
import org.gudy.azureus2.plugins.utils.LocaleUtilities;
import org.gudy.azureus2.plugins.utils.UTTimerEvent;
import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;

import edu.northwestern.swarmscreen.Main;
import edu.northwestern.swarmscreen.SSConfiguration;
import edu.northwestern.swarmscreen.util.HashMapCache;
import edu.northwestern.swarmscreen.util.HashSetCache;

/**
 * @author David Choffnes
 *
 */
public class TorrentFetcher implements UTTimerEventPerformer {

	private static final boolean TESTING = true;
	private static TorrentFetcher self;
	HashSetCache<URL> torrents = new HashSetCache<URL>(300);
//	private String testingURL = "https://isohunt.com/latest.php?mode=bt";
	private String testingURL = "http://linuxtracker.org/index.php?page=torrents&active=1tracker=0&order=finished&by=DESC";
	
	ArrayList<URL> torrentLocations = new ArrayList<URL>();
	private long maxDownloadTime = 30*1000;
	
	
	public static TorrentFetcher getInstance(){
		if (self == null){
			self = new TorrentFetcher();
			
		
				String filename = System.getProperty("java.home")
				+ "/lib/security/cacerts".replace('/', File.separatorChar);
				//String password = "maybeillaskusernextime";
				System.setProperty("javax.net.ssl.trustStore",filename);
				//System.setProperty("javax.net.ssl.trustStorePassword",password);
			
			
		}
		return self;
	}


	
	public void perform(UTTimerEvent event) {
		String torrentURLs[] = SSConfiguration.getInstance().getTorrentURLs();
		if (torrentURLs.length>0){
			synchronized(torrentLocations){
			torrentLocations.clear();
			for (String url : torrentURLs){
				try {
					torrentLocations.add(new URL(url));
				} catch (MalformedURLException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			}
		}
		int interval = SSConfiguration.getInstance().getTorrentDownloadInterval();
		if (!Main.isShuttingDown())
			Main.getInstance().getTimer("TorrentFetcher").addEvent(System.currentTimeMillis()+(long)(1000*(1+Math.random())*interval), this);
		
		// get torrents
		if (torrentLocations.size()==0){
			if (TESTING){
				try {
					synchronized(torrentLocations){
						torrentLocations.add(new URL(testingURL));
					}
				} catch (MalformedURLException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			} else {
				LocaleUtilities loc = Main.getPluginInterface().getUtilities().getLocaleUtilities();
				Main.getLogger().logAlert(LoggerChannel.LT_WARNING, 
						loc.getLocalisedMessageText("ss.noUrls") + 
						loc.getLocalisedMessageText("ss.wontWork"));
			}

		} else {
			synchronized (torrentLocations){
				for (URL location : torrentLocations) 
					torrents.addAll(getTorrents(location));
			}
		}
		TorrentManager.getInstance().loadTransientTorrents();
	}


	public Set<URL> getRandomTorrents() {
		return torrents;		
	}
	
	private List<URL> getTorrents(URL url){
		// TODO scrape torrents
		LinkedList<URL> toReturn = new LinkedList<URL>();
		
		try {
			String data = getResponse(url);
			
			data = data.replace("<", "\n");
			data = data.replace(">", "\n");
			String lines[] = data.split("\n");
			for (String line : lines){
				int start = line.indexOf("a href=\"");
				int end = line.indexOf(".torrent\"");
					
				if (start>=0 && end>=0){
					String torrentURL = line.substring(start+8, end+8);
					if (!torrentURL.contains("http")){
						String base = url.toString();
						base = URLDecoder.decode(base, "ISO-8859-1"); // TODO is there a way to figure out the encoding?
						if (base.contains("/")){
							base = base.substring(0, base.lastIndexOf("/"));
						} 
						torrentURL = base+(torrentURL.startsWith("/")?"":"/")+torrentURL;						
					}
					toReturn.add(new URL(torrentURL));
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if (toReturn.size()==0){
			LocaleUtilities loc = Main.getPluginInterface().getUtilities().getLocaleUtilities();
			Main.getLogger().logAlert(LoggerChannel.LT_WARNING, 
					loc.getLocalisedMessageText("ss.cantFind") + 
					url + "\n"+loc.getLocalisedMessageText("ss.wontWork"));
		}
		return toReturn;
	}
	
	private String getResponse(URL url) throws IOException {

		long startTime = System.currentTimeMillis();
		URLConnection urlConn = null;
		InputStream is = null;

		int count = 0;

		// URL connection channel.
		try {
			urlConn = url.openConnection();		
		} catch (IOException e2) {
			e2.printStackTrace();
			return null;
		}

		try {

			// Let the run-time system (RTS) know that we want input.
			urlConn.setDoInput(true);
			// Let the RTS know that we NOT want to do output.
			urlConn.setDoOutput(false);
			// No caching, we want the real thing.
			urlConn.setUseCaches(false);
			// Specify the content type.
			urlConn.setRequestProperty("Content-Type",
			"application/x-www-form-urlencoded");
			urlConn.setRequestProperty("accept-encoding",
			"gzip");

			urlConn.setConnectTimeout(5*1000);
			urlConn.connect();

			// Get response data.
			boolean useGzip = false;
			//System.out.println("Header: \n" + urlConn.getHeaderFields());
			for (Entry<String, List<String>> ent : urlConn.getHeaderFields().entrySet() ){
				for (String s : ent.getValue()){
					if (s.equals("gzip")) {
						useGzip = true;
						break;
					}
					if (useGzip) break;
				}
			}
			is = urlConn.getInputStream();
			if (useGzip) is = new GZIPInputStream(is);
			StringBuffer sb = new StringBuffer();
			byte b[] = new byte[1];
			boolean eof = false;
			int readCount = 0;
			while (!eof) {
				if (System.currentTimeMillis()-startTime>maxDownloadTime )break;
				int c = is.read(b);
				if (c < 0)
					break;
				sb.append((char) b[0]);
				readCount++;
//				if (readCount > 10000) {
//					System.out.println("Something wrong!?");
//					break;
//				}
			}
			is.close();
			return sb.toString();
		} finally {
			//	if (TESTING) System.out.println(" [Done]");
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
	
	public static void main(String args[]){
		// testing code
		getInstance();
		try {
			URL url = new URL("https://isohunt.com/latest.php?mode=bt");
			self.torrentLocations.add(url);
			self.getTorrents(url);
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void setURLs(ArrayList<String> urls){
		synchronized (torrentLocations){
			torrentLocations.clear();
			for (String url : urls){
				try {
					torrentLocations.add(new URL(url));
				} catch (MalformedURLException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
}
