/**
 * SwarmScreen Project
 *
 * File:         SSView.java
 * RCS:          $Id: SSView.java,v 1.13 2009/04/16 17:59:17 drc915 Exp $
 * Description:  SSView class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 16, 2007 at 3:33:47 PM
 * Language:     Java
 * Package:      edu.northwestern.swarmscreen.ui
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.swarmscreen.ui;


import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.gudy.azureus2.core3.config.COConfigurationManager;
import org.gudy.azureus2.core3.internat.MessageText;
import org.gudy.azureus2.core3.util.Debug;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.utils.LocaleUtilities;
import org.gudy.azureus2.plugins.utils.UTTimer;
import org.gudy.azureus2.plugins.utils.UTTimerEvent;
import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;
import org.gudy.azureus2.ui.swt.Messages;
import org.gudy.azureus2.ui.swt.Utils;
import org.gudy.azureus2.ui.swt.components.LinkLabel;
import org.gudy.azureus2.ui.swt.config.BooleanParameter;
import org.gudy.azureus2.ui.swt.config.DirectoryParameter;
import org.gudy.azureus2.ui.swt.config.IntParameter;
import org.gudy.azureus2.ui.swt.config.Parameter;
import org.gudy.azureus2.ui.swt.config.ParameterChangeListener;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEvent;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;
import org.gudy.azureus2.ui.swt.views.AbstractIView;

import edu.northwestern.swarmscreen.Main;
import edu.northwestern.swarmscreen.SSConfiguration;
import edu.northwestern.swarmscreen.connection.ConnectionProfiler;
import edu.northwestern.swarmscreen.torrents.TorrentFetcher;
import edu.northwestern.swarmscreen.util.Util;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The SSView class ...
 */
public class SSView extends AbstractIView implements UISWTViewEventListener {
  

private class UpdateThread extends Thread {
    boolean bContinue;
    SSView view;
    
		public UpdateThread(SSView view) {
			super("SwarmScreen Update Thread");
			this.view = view;
			
		}
    
    public void run() {
      try {
        bContinue = true;
        while(bContinue) {   
          
        	view.refreshActivity();

          
          Thread.sleep(1000);
        }
      } catch(Exception e) {
      	Debug.printStackTrace( e );  
      }
    }
    
    public void stopIt() {
      bContinue = false;
    }
  }



public static final String VIEWID = "SwarmScreen";


public static final String EDGE_MAPPINGS = "EdgeMappings";
public static final String NEARBY_PEERS = "NearbyPeers";

/** in seconds */
private static final int PERIODIC_UPDATE_INTERVAL = 5;


private static final String CFG_PREFIX = "ss.ui.";



private static double myRatio = -1;


private static double mySimilarity = -1;
  
	ScrolledComposite scrollPanel;
Composite panel;
  boolean activityChanged;
  Table activityTable;
  Table edgeTable;
  private boolean isCreated = false;
  UpdateThread updateThread;
protected boolean ipsChanged;
private PluginInterface pi;
private UTTimer timer;
boolean listenersAdded = false;
private Table alertTable;
private boolean noChange;
private Label currentRatio;
private Label currentSim;


private Group gStatus;


private Scale scale;


private Label scaleValueLabel;

  public SSView( PluginInterface pi ) {
   this.pi = pi;
    init();
  }
  
  private void init() {

	    activityChanged = true;
  }
  
  public void initialize(Composite composite) {
	  LocaleUtilities loc = Main.getPluginInterface().getUtilities().getLocaleUtilities();

	  scrollPanel = new ScrolledComposite(composite,SWT.V_SCROLL|SWT.H_SCROLL);
	  scrollPanel.setExpandHorizontal(true);
	  scrollPanel.setExpandVertical(true);
	  
	  
	  panel = new Composite(scrollPanel, SWT.NONE);	  

    GridLayout layout = new GridLayout();
    layout.numColumns = 2;
    
    panel.setLayout(layout);
    
    timer = pi.getUtilities().createTimer("SwarmScreenViewUpdateTimer");

    timer.addPeriodicEvent(PERIODIC_UPDATE_INTERVAL*1000, new UTTimerEventPerformer(){

		public void perform(UTTimerEvent event) {
			periodicUpdate();
			
		}});    
    
//    Label label = new Label(panel, SWT.WRAP);
//    Messages.setLanguageText(label,"DetailsPane.text.main");
//    Label label2 = new Label(panel, SWT.WRAP);
//    Messages.setLanguageText(label2,"DetailsPane.text.feedback");
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);

    gridData = new GridData();
	gridData.horizontalSpan = 2;
	gridData.widthHint=450;
	new LinkLabel(panel, gridData, "SSView.label.forums",
	"http://forums.aqua-lab.org");
	new LinkLabel(panel, gridData, "SSView.label.aqualab",
			"http://aqualab.cs.northwestern.edu");

    
	final BooleanParameter enable = new BooleanParameter(panel,	
			"ss.enable", 
			"ss.enable");
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	enable.setLayoutData(gridData);
	enable.setValue(SSConfiguration.getInstance().isEnabled());
	
	
    final Group gSettings = new Group(panel, SWT.NULL);
	Messages.setLanguageText(gSettings, CFG_PREFIX + "settings.group");
	gridData = new GridData(GridData.BEGINNING);
	gSettings.setLayoutData(gridData);
//	GridLayout layout = new GridLayout();
//	layout.numColumns = 2;
	gSettings.setLayout(layout);
	if (!SSConfiguration.getInstance().isEnabled()) gSettings.setEnabled(false);
	
	
	
	final Label torrentDirLabel = new Label(gSettings, SWT.WRAP);
	torrentDirLabel.setText(loc.getLocalisedMessageText("ss.torrentDirLabel"));
	
	Composite c = new Composite(gSettings, SWT.NULL);	 
	layout = new GridLayout();
    layout.numColumns = 2;  
	c.setLayout(layout);
	String torrentDirS = SSConfiguration.getInstance().getTorrentDataDir();
	if (torrentDirS==null || torrentDirS.length()==0 || torrentDirS.equals("null")) 
		torrentDirS = loc.getLocalisedMessageText("ss.torrentDirTextbox");
	final DirectoryParameter torrentDir = 
		new DirectoryParameter(c, "ss.torrentDirLabel",  torrentDirS);				
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	gridData.widthHint=350;
	gridData.grabExcessVerticalSpace = true;
	c.setLayoutData(gridData);

	
	
	

	// add textbox and list for URLs
	final Label urlLabel = new Label(gSettings, SWT.WRAP);
	urlLabel.setText(loc.getLocalisedMessageText("ss.enterUrlLabel"));
	final Text urlTextbox = new Text(gSettings, SWT.SINGLE|SWT.BORDER);
	urlTextbox.setTextLimit(350);
	urlTextbox.setText(loc.getLocalisedMessageText("ss.enterUrl"));
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	gridData.widthHint=350;
	urlTextbox.setLayoutData(gridData);
	
	final Label urlListLabel = new Label(gSettings, SWT.WRAP);
	urlListLabel.setText(loc.getLocalisedMessageText("ss.urlListLabel"));
	final List urlList = new List(gSettings, SWT.V_SCROLL|SWT.H_SCROLL|SWT.BORDER);
	urlList.setSize(200, 50);
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	gridData.heightHint=100;
	gridData.widthHint=350;
	urlList.setLayoutData(gridData);
	if (SSConfiguration.getInstance().getTorrentURLs().length>0){
		for (String s : SSConfiguration.getInstance().getTorrentURLs())
			urlList.add(s);
	}
//	final Label removeLabel = new Label(gSettings, SWT.WRAP);
//	removeLabel.setText(loc.getLocalisedMessageText("ss.urlListRemove"));
	
	
	/////////////////////////////
	// Right side
	gStatus = new Group(panel, SWT.NULL);
	Messages.setLanguageText(gStatus, CFG_PREFIX + "status.group");
	gridData = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
//	gridData.heightHint=200;
//	gridData.horizontalSpan = 2;
	gStatus.setLayoutData(gridData);	
//	GridLayout layout = new GridLayout();
//	layout.numColumns = 2;
	gStatus.setLayout(layout);

	final Label scaleLabel = new Label(gStatus, SWT.WRAP);
	int privacyLevel = SSConfiguration.getInstance().getPrivacyLevel();
	scaleLabel.setText(loc.getLocalisedMessageText("ss.privacySetting")+":");	
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	scaleLabel.setLayoutData(gridData);
	
	final Composite c2 = new Composite(gStatus, SWT.NULL);	 
	layout = new GridLayout();
    layout.numColumns = 3;  
	c2.setLayout(layout);
	scaleValueLabel = new Label(c2, SWT.WRAP|SWT.BORDER);
	scaleValueLabel.setAlignment(SWT.CENTER);
	scaleValueLabel.setText(privacyLevel+"");
	FontData existing = scaleValueLabel.getFont().getFontData()[0];
	FontData labelFontData = new FontData(existing.getName(), existing.getHeight(), SWT.BOLD);
	Font labelFont = new Font(panel.getDisplay(), labelFontData);	
	scaleValueLabel.setFont(labelFont);
	gridData = new GridData();
	gridData.widthHint = 30;
	scaleValueLabel.setLayoutData(gridData);
	
	scale  =
		   new Scale(c2,SWT.HORIZONTAL);
	scale.setMinimum(0);
	scale.setMaximum(200);
	scale.setIncrement(5);
	scale.setPageIncrement(10);
	scale.setSelection(SSConfiguration.getInstance().getPrivacyLevel());
//	gridData = new GridData();
//	gridData.horizontalAlignment = GridData.HORIZONTAL_ALIGN_CENTER;
//	scale.setLayoutData(gridData);
	
	final Label scaleMaxLabel = new Label(c2, SWT.WRAP);
	scaleMaxLabel.setText(loc.getLocalisedMessageText("ss.privacyMax"));	
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	gridData.widthHint=400;
	gridData.grabExcessVerticalSpace = true;
	c2.setLayoutData(gridData);
	
	 
	
	
	currentRatio = new Label(gStatus, SWT.WRAP);
	currentRatio.setText(loc.getLocalisedMessageText("ss.currentRatio")+": "+"["+
			loc.getLocalisedMessageText("ss.unknown")+"]");	
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	currentRatio.setLayoutData(gridData);	
	
	Label targetRatio = new Label(gStatus, SWT.WRAP|SWT.RIGHT);
	targetRatio.setText(loc.getLocalisedMessageText("ss.targetRatioValue"));	
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	targetRatio.setLayoutData(gridData);	
	
	currentSim = new Label(gStatus, SWT.WRAP);
	currentSim.setText(loc.getLocalisedMessageText("ss.currentSimilarity")+": "+"["+
			loc.getLocalisedMessageText("ss.unknown")+"]");	
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	currentSim.setLayoutData(gridData);	
	currentSim.setVisible(false); // until ready
	
	Label maxFileSizeLabel = new Label(gStatus, SWT.NULL);
	Messages.setLanguageText(maxFileSizeLabel, "ss.ui.maxFileSize");
		final IntParameter maxFileSize = new IntParameter(gStatus, CFG_PREFIX + "maxFileSize", 
			1, Integer.MAX_VALUE);
	maxFileSize.getControl().setEnabled(enable.isSelected());
	maxFileSize.setValue(SSConfiguration.getInstance().getMaxFileSize());
	

	if (!SSConfiguration.getInstance().isEnabled()){
		gSettings.setEnabled(false);
		urlTextbox.setEnabled(false);
		urlList.setEnabled(false);
		scale.setEnabled(false);
		for (Control cl : torrentDir.getControls()){
			cl.setEnabled(false);
		}
	}
	
	///////////////////////////////////////
	// Actions
	
	maxFileSize.addChangeListener(new ParameterChangeListener(){

		
		public void booleanParameterChanging(Parameter p, boolean toValue) {
			// TODO Auto-generated method stub
			
		}

		
		public void floatParameterChanging(Parameter owner, double toValue) {
			// TODO Auto-generated method stub
			
		}

		
		public void intParameterChanging(Parameter p, int toValue) {
			SSConfiguration.getInstance().setMaxFileSize(toValue);
			Main.saveProperties();
		}

		
		public void parameterChanged(Parameter p, boolean caused_internally) {
			// TODO Auto-generated method stub
			
		}

		
		public void stringParameterChanging(Parameter p, String toValue) {
			// TODO Auto-generated method stub
			
		}});
	
	scale.addSelectionListener(
			  new SelectionAdapter()
			  {
			    public void widgetSelected(SelectionEvent e)
			    {
			    	LocaleUtilities loc = Main.getPluginInterface().getUtilities().getLocaleUtilities();
			    	scaleValueLabel.setText(scale.getSelection()+"");	
			    	
			    	SSConfiguration.getInstance().setPrivacyLevel(scale.getSelection());
			    	c2.layout();
			    	Main.saveProperties();
			    }
			  }
	);
	
	torrentDir.addChangeListener(new ParameterChangeListener(){

		
		public void booleanParameterChanging(Parameter p, boolean toValue) {
			// TODO Auto-generated method stub
			
		}

		
		public void floatParameterChanging(Parameter owner, double toValue) {
			// TODO Auto-generated method stub
			
		}

		
		public void intParameterChanging(Parameter p, int toValue) {
			// TODO Auto-generated method stub
			
		}

		
		public void parameterChanged(Parameter p, boolean caused_internally) {
			String s = COConfigurationManager.getStringParameter("ss.torrentDirLabel");
			SSConfiguration.getInstance().setTorrentDataDir(s);
			Main.saveProperties();
		}

		
		public void stringParameterChanging(Parameter p, String toValue) {
			
			
		}});
	
	enable.addChangeListener(new ParameterChangeListener(){

		
		public void booleanParameterChanging(Parameter p, boolean toValue) {
			SSConfiguration.getInstance().setEnabled(toValue);	
			gSettings.setEnabled(toValue);
			urlTextbox.setEnabled(toValue);
			urlList.setEnabled(toValue);
			scale.setEnabled(toValue);
			maxFileSize.getControl().setEnabled(toValue);
			for (Control c : torrentDir.getControls()){
				c.setEnabled(toValue);
			}
			Main.saveProperties();
		}

		
		public void floatParameterChanging(Parameter owner, double toValue) {
			// TODO Auto-generated method stub
			
		}

		
		public void intParameterChanging(Parameter p, int toValue) {
			// TODO Auto-generated method stub
			
		}

		
		public void parameterChanged(Parameter p, boolean caused_internally) {
			// TODO Auto-generated method stub
			
		}

		
		public void stringParameterChanging(Parameter p, String toValue) {
			// TODO Auto-generated method stub
			
		}});
	
	urlTextbox.addKeyListener(new KeyListener(){

		
		public void keyPressed(KeyEvent arg0) {
			
		}

		
		public void keyReleased(KeyEvent arg0) {
			if (arg0.character=='\n' || arg0.character=='\r'){
				urlList.add(urlTextbox.getText());
				urlTextbox.setText("");
				SSConfiguration.getInstance().setTorrentURLs(urlList.getItems());
				TorrentFetcher.getInstance().perform(null);
				Main.saveProperties();
			}
			
			
		}});
	urlTextbox.addFocusListener(new FocusListener(){

		
		public void focusGained(FocusEvent arg0) {
			LocaleUtilities loc = Main.getPluginInterface().getUtilities().getLocaleUtilities();
			if (urlTextbox.getText().equals(loc.getLocalisedMessageText("ss.enterUrl"))){
				urlTextbox.setText("");
			}
			
		}

		
		public void focusLost(FocusEvent arg0) {
			
		}});
	
	urlList.addMouseListener(new MouseListener(){

		
		public void mouseDoubleClick(MouseEvent arg0) {
			for (String s : urlList.getSelection()){
				urlList.remove(s);
				SSConfiguration.getInstance().setTorrentURLs(urlList.getItems());
			}
			
		}

		
		public void mouseDown(MouseEvent arg0) {
			
		}

	
		public void mouseUp(MouseEvent arg0) {
			
		}});
	
	scrollPanel.setContent(panel);
	scrollPanel.setMinSize(800, 600);

  }
  

  public String getFullTitle() {
    return MessageText.getString("SSView.title.full");
  }
  
  public Composite getComposite() {
    return panel;
  }

  
  private void refreshActivity() {

  }
  


  
  public void periodicUpdate() {

  }
  
  

public String getData() {
    return "DHTView.title.full";
  }

	public boolean eventOccurred(UISWTViewEvent event) {
		switch (event.getType()) {
		case UISWTViewEvent.TYPE_CREATE:
	        if (isCreated){
	        	noChange = false;
	        	refresh();
	          return false;
	        }
	        noChange = false;
	        isCreated  = true;
	        break;
	        
		 case UISWTViewEvent.TYPE_DESTROY:
		        delete();
		        isCreated = false;
		        break;
		        
		        
		 case UISWTViewEvent.TYPE_INITIALIZE:
		        initialize((Composite)event.getData());
		        break;

	      case UISWTViewEvent.TYPE_LANGUAGEUPDATE:
	        //updateLanguage();
	        break;

	      case UISWTViewEvent.TYPE_REFRESH:
	        refresh();
	        break;
	    }

	    return true;
	}


	private void initializeUI(Composite composite) {
		System.out.println("Initializing UI!");
		
	}

	public void delete() {
		timer.destroy();
	    Utils.disposeComposite(panel);
	    
		
	}
    
	
	public void refresh() {
		LocaleUtilities loc = Main.getPluginInterface().getUtilities().getLocaleUtilities();
		double ratio = ConnectionProfiler.getInstance().getConnectionTimesRatio();
		currentRatio.setText(loc.getLocalisedMessageText("ss.currentRatio")+": "+
				(ratio>0?Util.formatDouble(ratio):"["+
				loc.getLocalisedMessageText("ss.unknown")+"]"));
		String simText = loc.getLocalisedMessageText("ss.unknown");
		
		double sim = ConnectionProfiler.getInstance().getConnectionDistributionSimilarity();
		if (sim>0.05){
			simText = loc.getLocalisedMessageText("ss.similarity.notSimilar");
		}else if (sim>0.01){
			simText = loc.getLocalisedMessageText("ss.similarity.similar");
		} else if (sim <=0.01 && sim >=0)  simText = loc.getLocalisedMessageText("ss.similarity.verySimilar");
		currentSim.setText(loc.getLocalisedMessageText("ss.currentSimilarity")+": "+
				"["+simText+"]");	
		gStatus.layout();
		
		if (scale.getSelection()!=SSConfiguration.getInstance().getPrivacyLevel()){
			scale.setSelection(SSConfiguration.getInstance().getPrivacyLevel());	
			scaleValueLabel.setText(""+SSConfiguration.getInstance().getPrivacyLevel());
		}
		
		if (alertTable == null || alertTable.isDisposed() || noChange)
	      return;

	    noChange = true;
	      

	
	  }



	  public static void updateRatio(double ratio){
		  myRatio = ratio;
	  }

	  public static void updateSimilarity(double similarity){
		  mySimilarity = similarity;
	  }
}


